<?xml version="1.0" encoding="ISO-8859-1"?>

<!--  Copyright 2010,2015 EUROPEAN UNION
  Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
  the European Commission - subsequent versions of the EUPL (the "Licence");
  You may not use this work except in compliance with the Licence.
  You may obtain a copy of the Licence at:
 
  http://ec.europa.eu/idabc/eupl
 
  Unless required by applicable law or agreed to in writing, software
  distributed under the Licence is distributed on an "AS IS" basis,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the Licence for the specific language governing permissions and
  limitations under the Licence.
 
  Date: 15/07/2016
  Authors: European Commission, Joint Research Centre
  Daniele Francioli, Emanuela Epure
  inspire-registry-dev@jrc.ec.europa.eu
 
This work was supported by the EU  Interoperability Solutions for
European Public Administrations Programme (http://ec.europa.eu/isa)
through Action 1.17: Re-usable INSPIRE Reference Platform 
(http://ec.europa.eu/isa/actions/01-trusted-information-exchange/1-17action_en.htm).-->

<xsl:stylesheet version="2.0" 
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
                xmlns:gmd="http://www.isotc211.org/2005/gmd"
                xmlns:gco="http://www.isotc211.org/2005/gco"
                xmlns="http://www.isotc211.org/2005/grg"
> 
    
    <!--Output settings-->
    <xsl:output method="xml" media-type="application/xml" version="1.0" encoding="UTF-8" indent="yes"/>

    <!--Variables & parameters declaration-->
    <xsl:param name="base_db_export_path" />
    
    <xsl:template match="/">
        <RE_Register 
            xmlns="http://www.isotc211.org/2005/grg" 
            xsi:schemaLocation="http://www.isotc211.org/2005/grg http://standards.iso.org/iso/19135/-2/reg/1.0/registration.xsd">
            <xsl:apply-templates select="registry"/>
        </RE_Register>
    </xsl:template>
    
    <xsl:template match="registry">     
        
        <!--Name-->
        <xsl:element name="name">
            <gco:CharacterString>
                <xsl:value-of select="label" /> 
            </gco:CharacterString>           
        </xsl:element>
        
        <!--Content summary-->
        <xsl:element name="contentSummary" >
            <xsl:element name="gco:CharacterString" >
                <xsl:value-of select="definition"/>
                <xsl:if test="description != ''">
                    <xsl:value-of select="description"/>
                </xsl:if>
            </xsl:element>  
        </xsl:element>  
        
        <!--Registry URI-->
        <xsl:element name="uniformResourceIdentifier">
            <xsl:element name="gmd:CI_OnlineResource">
                <xsl:element name="gmd:linkage">
                    <xsl:element name="gmd:URL">
                        <xsl:value-of select="uri"/>
                    </xsl:element>  
                </xsl:element>
            </xsl:element>
        </xsl:element>
        
        <!--Operating language-->
        <xsl:element name="operatingLanguage">
            <xsl:element name="RE_Locale">
                <xsl:element name="name">
                    <xsl:element name="gco:CharacterString">
                        <xsl:value-of select="language/label"/>
                    </xsl:element>
                </xsl:element>
                <xsl:element name="language">
                    <xsl:element name="gmd:LanguageCode">
                        <xsl:attribute name="codeList">
                            <xsl:text>http://www.loc.gov/standards/iso639-2</xsl:text>
                        </xsl:attribute>
                        <xsl:attribute name="codeListValue">
                            <xsl:value-of select="language/iso639_2"/>
                        </xsl:attribute>
                    </xsl:element>
                </xsl:element>
                <xsl:element name="country">
                    <xsl:attribute name="gco:nilReason">
                        <xsl:text>inapplicable</xsl:text>
                    </xsl:attribute>            
                </xsl:element>                                   
                <xsl:element name="characterEncoding">
                    <xsl:element name="gmd:MD_CharacterSetCode">
                        <xsl:attribute name="codeList">
                            <xsl:text>http://www.isotc211.org/2005/resources/Codelist/gmxCodelists.xml</xsl:text>
                        </xsl:attribute>
                        <xsl:attribute name="codeListValue">
                            <xsl:text>utf8</xsl:text>
                        </xsl:attribute>
                    </xsl:element>
                </xsl:element>                    
                <xsl:element name="citation">
                    <xsl:attribute name="gco:nilReason">
                        <xsl:text>inapplicable</xsl:text>
                    </xsl:attribute>            
                </xsl:element>
            </xsl:element>
        </xsl:element>
        
        <!--Alternative languages-->              
        <xsl:element name="alternativeLanguages">
            <xsl:attribute name="gco:nilReason">
                <xsl:text>inapplicable</xsl:text>
            </xsl:attribute>            
        </xsl:element>
        
        <!--Register owner-->
        <xsl:element name="owner">
            <xsl:attribute name="gco:nilReason">
                <xsl:text>inapplicable</xsl:text>
            </xsl:attribute>            
        </xsl:element>
        
        <!--Register submitter-->
        <xsl:element name="submitter">
            <xsl:attribute name="gco:nilReason">
                <xsl:text>inapplicable</xsl:text>
            </xsl:attribute>            
        </xsl:element> 
        
        <!--ItemClass-->     
        <xsl:element name="containedItemClass">
            <xsl:element name="RE_ItemClass">
                <xsl:element name="name">
                    <xsl:element name="gco:CharacterString">
                        <xsl:text>register</xsl:text>
                    </xsl:element>			
                </xsl:element>
                <xsl:element name="technicalStandard">
                    <xsl:attribute name="gco:nilReason">
                        <xsl:text>inapplicable</xsl:text>
                    </xsl:attribute>            
                </xsl:element>
                <xsl:element name="alternativeNames">
                    <xsl:attribute name="gco:nilReason">
                        <xsl:text>inapplicable</xsl:text>
                    </xsl:attribute>            
                </xsl:element>
                <xsl:element name="describedItem">
                    <xsl:attribute name="gco:nilReason">
                        <xsl:text>inapplicable</xsl:text>
                    </xsl:attribute>            
                </xsl:element>
            </xsl:element>
        </xsl:element>
        
        <!--Registry manager-->
        <xsl:choose>
            <xsl:when test="registrymanager != ''">
                <xsl:element name="manager">          
                    <xsl:element name="RE_RegisterManager">
                        <xsl:element name="name">
                            <xsl:element name="gco:CharacterString">
                                <xsl:value-of select="registrymanager/label" />
                            </xsl:element>  
                        </xsl:element>
                        <xsl:element name="contact">                    
                            <xsl:element name="gmd:CI_ResponsibleParty">
                                <xsl:element name="gmd:role">
                                    <xsl:element name="gmd:CI_RoleCode">
                                        <xsl:attribute name="codeList">
                                            <xsl:text>http://www.isotc211.org/2005/resources/Codelist/gmxCodelists.xml#CI_RoleCode</xsl:text>
                                        </xsl:attribute>
                                        <xsl:attribute name="codeListValue">
                                            <xsl:text>custodian</xsl:text>
                                        </xsl:attribute>
                                    </xsl:element>
                                </xsl:element>
                            </xsl:element>                    
                        </xsl:element>
                    </xsl:element>
                </xsl:element> 
            </xsl:when>
            <xsl:otherwise>
                <xsl:element name="manager">
                    <xsl:attribute name="gco:nilReason">
                        <xsl:text>inapplicable</xsl:text>
                    </xsl:attribute>            
                </xsl:element>
            </xsl:otherwise>
        </xsl:choose>    
   
        <!--Registers-->
        <xsl:choose>
            <xsl:when test="registers">

                <xsl:for-each select="registers/register">
                
                    <xsl:element name="containedItem" >
                        <xsl:element name="RE_RegisterItem" >
                            
                            <xsl:element name="itemIdentifier">
                                <xsl:attribute name="gco:nilReason">
                                    <xsl:text>inapplicable</xsl:text>
                                </xsl:attribute>            
                            </xsl:element>
                            
                            <!--Name-->
                            <xsl:element name="name">
                                <gco:CharacterString>
                                    <xsl:value-of select="label" /> 
                                </gco:CharacterString>           
                            </xsl:element>
                            
                            <!--Status-->
                            <xsl:element name="status">
                                <xsl:attribute name="gco:nilReason">
                                    <xsl:text>inapplicable</xsl:text>
                                </xsl:attribute>            
                            </xsl:element>
                            
                            <!--Definition-->
                            <xsl:choose>
                                <xsl:when test="definition != ''">
                                    <xsl:element name="definition">
                                        <xsl:element name="gco:CharacterString">
                                            <xsl:value-of select="definition"/>
                                        </xsl:element>			
                                    </xsl:element>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:element name="definition">
                                        <xsl:attribute name="gco:nilReason">
                                            <xsl:text>missing</xsl:text>
                                        </xsl:attribute>            
                                    </xsl:element>
                                </xsl:otherwise>
                            </xsl:choose> 
                            
                            <!--Description-->
                            <xsl:choose>
                                <xsl:when test="description != ''">
                                    <xsl:element name="description">
                                        <xsl:element name="gco:CharacterString">
                                            <xsl:value-of select="descriptions"/>
                                        </xsl:element>			
                                    </xsl:element>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:element name="description">
                                        <xsl:attribute name="gco:nilReason">
                                            <xsl:text>missing</xsl:text>
                                        </xsl:attribute>            
                                    </xsl:element>
                                </xsl:otherwise>
                            </xsl:choose>
                            
                            <!--Additional -->
                            <xsl:element name="additionInformation">
                                <xsl:attribute name="gco:nilReason">
                                    <xsl:text>inapplicable</xsl:text>
                                </xsl:attribute>            
                            </xsl:element>
                            
                            <!--Itemclass-->
                            <xsl:element name="itemClass">
                                <xsl:attribute name="gco:nilReason">
                                    <xsl:text>register</xsl:text>
                                </xsl:attribute>            
                            </xsl:element>                    
                    
                        </xsl:element>
                    </xsl:element>   
                </xsl:for-each>
       
            </xsl:when>
            <xsl:otherwise>
            </xsl:otherwise>
        </xsl:choose>
        
    </xsl:template>
</xsl:stylesheet>