<?xml version="1.0" encoding="ISO-8859-1"?>

<!--  Copyright 2010,2015 EUROPEAN UNION
  Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
  the European Commission - subsequent versions of the EUPL (the "Licence");
  You may not use this work except in compliance with the Licence.
  You may obtain a copy of the Licence at:
 
  http://ec.europa.eu/idabc/eupl
 
  Unless required by applicable law or agreed to in writing, software
  distributed under the Licence is distributed on an "AS IS" basis,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the Licence for the specific language governing permissions and
  limitations under the Licence.
 
  Date: 15/07/2016
  Authors: European Commission, Joint Research Centre
  Daniele Francioli, Emanuela Epure
  inspire-registry-dev@jrc.ec.europa.eu
 
This work was supported by the EU  Interoperability Solutions for
European Public Administrations Programme (http://ec.europa.eu/isa)
through Action 1.17: Re-usable INSPIRE Reference Platform 
(http://ec.europa.eu/isa/actions/01-trusted-information-exchange/1-17action_en.htm).-->

<xsl:stylesheet version="2.0" 
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
                xmlns="http://inspire.ec.europa.eu/enumeration_register/enumeration"> 
       
    <!--Output settings-->
    <xsl:output method="xml" media-type="application/xml" version="1.0" encoding="UTF-8" indent="yes"/>

    <!--Variables & parameters declaration-->
    <xsl:param name="base_db_export_path" />
    
    <xsl:template match="/">
        <enumeration  xsi:schemaLocation="http://inspire.ec.europa.eu/enumeration_register/enumeration http://inspire.ec.europa.eu/draft-schemas/registry/1.2/Enumeration.xsd">
            <xsl:apply-templates select="item"/>
        </enumeration>
    </xsl:template>
    
    <xsl:template match="item">        
        <xsl:attribute name="id">
            <xsl:value-of select="uri"/>
        </xsl:attribute>  
        
        <xsl:if test="thisversion != ''">
            <xsl:element name="thisversion" >
                <xsl:value-of select="thisversion/uri" />
            </xsl:element>
        </xsl:if>
        
        <xsl:if test="latestversion != ''">
            <xsl:element name="latestversion" >
                <xsl:value-of select="latestversion/uri" />
            </xsl:element>
        </xsl:if>
          
        <xsl:if test="previousversions != ''">
            <xsl:element name="previousversions" >
                <xsl:for-each select="previousversions/version">
                    <xsl:element name="version">
                        <xsl:value-of select="uri"/>         
                    </xsl:element>                
                </xsl:for-each>
            </xsl:element>
        </xsl:if>
        
        <xsl:variable name="language" select="language/isocode"></xsl:variable>  
        <xsl:element name="language" >
            <xsl:value-of select="$language" />
        </xsl:element>
          
        <xsl:if test="label != ''">
            <xsl:element name="label" >
                <xsl:attribute name="xml:lang">
                    <xsl:value-of select="label/@language"/>
                </xsl:attribute>
                <xsl:value-of select="label" />
            </xsl:element>
        </xsl:if>
        
        <xsl:if test="definition != ''">
            <xsl:variable name="tmp" select="definition"/>
            
            <xsl:element name="definition" >
                <xsl:attribute name="xml:lang">
                    <xsl:value-of select="definition/@language"/>
                </xsl:attribute>
                <xsl:value-of select="$tmp" />
            </xsl:element>
        </xsl:if>
        
        <xsl:if test="description != ''">
            <xsl:variable name="tmp" select="description"/>
            
            <xsl:element name="description" >
                <xsl:attribute name="xml:lang">
                    <xsl:value-of select="description/@language"/>
                </xsl:attribute>
                <xsl:value-of select="$tmp" />
            </xsl:element>        
        </xsl:if>
        
        
        <xsl:variable name="registryUri" select="register/registry/uri" />
        <xsl:if test="customattributes/customattribute[name/text()='Extensibility'] != ''">            
            <xsl:if test="customattributes/customattribute[name/text()='Extensibility']/values != ''">            
                <xsl:for-each select="customattributes/customattribute[name/text()='Extensibility']/values/value">                   
                    <xsl:variable name="extensibility_label" select="uriname"></xsl:variable>
                    <xsl:variable name="extensibility_url" select="concat($registryUri,'/extensibility/', $extensibility_label)"/>
                    <xsl:if test="$extensibility_label != ''">
                        <xsl:element name="extensibility" >
                            <xsl:attribute name="id">
                                <xsl:value-of select="$extensibility_url"/>
                            </xsl:attribute>
            
                            <xsl:element name="uriname" >
                                <xsl:value-of select="$extensibility_label" />
                            </xsl:element>
                        </xsl:element>
                    </xsl:if>            
                </xsl:for-each>
            </xsl:if>
        </xsl:if>
        
        <xsl:if test="customattributes/customattribute[name/text()='GovernanceLevel'] != ''">            
            <xsl:if test="customattributes/customattribute[name/text()='GovernanceLevel']/values != ''">            
                <xsl:for-each select="customattributes/customattribute[name/text()='GovernanceLevel']/values/value">                   
                    <xsl:variable name="governance_uriname" select="uriname"/>
                    <xsl:variable name="governance_url" select="concat($registryUri,'/governance-level/', $governance_uriname)"/>
                    <xsl:if test="$governance_uriname != ''">                    
                        <xsl:element name="governance-level" >
                            <xsl:attribute name="id">
                                <xsl:value-of select="$governance_url"/>
                            </xsl:attribute>
                            <xsl:element name="label" >
                                <xsl:value-of select="$governance_uriname"/>
                            </xsl:element>    
                        </xsl:element>  
                    </xsl:if>              
                </xsl:for-each>
            </xsl:if>
        </xsl:if>
        
                            
        <xsl:if test="customattributes/customattribute[name/text()='Theme'] != ''">            
            <xsl:if test="customattributes/customattribute[name/text()='Theme']/values != ''">            
                <xsl:for-each select="customattributes/customattribute[name/text()='Theme']/values/value">                   
                    <xsl:variable name="themeUri" select="uri"></xsl:variable>
                    <xsl:element name="theme">
                        <xsl:attribute name="id">
                            <xsl:value-of select="$themeUri"/>
                        </xsl:attribute>
                        
                        <xsl:if test="label">
                            <xsl:variable name="themeLabel" select="label"></xsl:variable>
                            <xsl:variable name="themeLabelLanguage" select="label/@language"></xsl:variable>
                            <label>
                                <xsl:attribute name="xml:lang">
                                    <xsl:value-of select="$themeLabelLanguage"/>
                                </xsl:attribute>
                                <xsl:value-of select="$themeLabel"/>
                            </label>
                        </xsl:if>
                    </xsl:element>              
                </xsl:for-each>
            </xsl:if>
        </xsl:if>
        
        <xsl:if test="customattributes/customattribute[name/text()='ApplicationSchema'] != ''">            
            <xsl:if test="customattributes/customattribute[name/text()='ApplicationSchema']/values != ''">            
                <xsl:for-each select="customattributes/customattribute[name/text()='ApplicationSchema']/values/value">                   
                    <xsl:variable name="applicationschemaUri" select="uri"></xsl:variable>
                    <xsl:element name="applicationschema">
                        <xsl:attribute name="id">
                            <xsl:value-of select="$applicationschemaUri"/>
                        </xsl:attribute>
                        
                        <xsl:if test="label">
                            <xsl:variable name="applicationschemaLabel" select="label"></xsl:variable>
                            <xsl:variable name="applicationschemaLabelLanguage" select="label/@language"></xsl:variable>
                            <label>
                                <xsl:attribute name="xml:lang">
                                    <xsl:value-of select="$applicationschemaLabelLanguage"/>
                                </xsl:attribute>
                                <xsl:value-of select="$applicationschemaLabel"/>
                            </label>
                        </xsl:if>
                    </xsl:element>              
                </xsl:for-each>
            </xsl:if>
        </xsl:if> 
        
        <xsl:element name="itemclass" >
            <xsl:attribute name="uriname">
                <xsl:value-of select="itemclass/uriname"/>
            </xsl:attribute>
            
            <xsl:element name="label" >
                <xsl:attribute name="xml:lang">
                    <xsl:value-of select="itemclass/label/@language"/>
                </xsl:attribute>
                <xsl:value-of select="itemclass/label" />
            </xsl:element>
        </xsl:element>
        
        <xsl:element name="status" >
            <xsl:attribute name="id">
                <xsl:value-of select="status/baseuri"/>
                <xsl:text>/</xsl:text>
                <xsl:value-of select="status/uriname"/>
            </xsl:attribute>
            
            <xsl:element name="label" >
                <xsl:attribute name="xml:lang">
                    <xsl:value-of select="status/label/@language"/>
                </xsl:attribute>
                <xsl:value-of select="status/label" />
            </xsl:element>
        </xsl:element>
       
        <xsl:element name="register" >
            <xsl:attribute name="id">
                <xsl:value-of select="register/uri"/>
            </xsl:attribute>
            
            <label>
                <xsl:attribute name="xml:lang">
                    <xsl:value-of select="register/label/@language"/>
                </xsl:attribute>
                <xsl:value-of select="register/label"/>
            </label>
                                
            <xsl:element name="registry" >
                <xsl:attribute name="id">
                    <xsl:value-of select="register/registry/uri"/>
                </xsl:attribute>
            
                <label>
                    <xsl:attribute name="xml:lang">
                        <xsl:value-of select="register/registry/label/@language"/>
                    </xsl:attribute>
                    <xsl:value-of select="register/registry/label"/>
                </label>
            </xsl:element>
        </xsl:element>   
        
        <xsl:choose>
            <xsl:when test="parents">
                <xsl:element name="parents" >
                    <xsl:for-each select="parents/parent">
                
                        <xsl:element name="parent">
                            <xsl:attribute name="id">
                                <xsl:value-of select="uri"/>
                            </xsl:attribute>         
                            
                            <xsl:if test="label != ''">
                                <xsl:element name="label" >
                                    <xsl:attribute name="xml:lang">
                                        <xsl:value-of select="label/@language"/>
                                    </xsl:attribute>
                                    <xsl:value-of select="label" />
                                </xsl:element>
                            </xsl:if>
                                              
                        </xsl:element>                
                    </xsl:for-each>
                </xsl:element>
            </xsl:when>
            <xsl:otherwise>
            </xsl:otherwise>
        </xsl:choose>
        
        <xsl:choose>
            <xsl:when test="predecessors">
                <xsl:element name="predecessors" >
                    <xsl:for-each select="predecessors/predecessor">
                
                        <xsl:element name="predecessor">
                            <xsl:attribute name="id">
                                <xsl:value-of select="uri"/>
                            </xsl:attribute>         
                            
                            <xsl:if test="label != ''">
                                <xsl:element name="label" >
                                    <xsl:attribute name="xml:lang">
                                        <xsl:value-of select="label/@language"/>
                                    </xsl:attribute>
                                    <xsl:value-of select="label" />
                                </xsl:element>
                            </xsl:if>
                                              
                        </xsl:element>                
                    </xsl:for-each>
                </xsl:element>
            </xsl:when>
            <xsl:otherwise>
            </xsl:otherwise>
        </xsl:choose>
        
        <xsl:choose>
            <xsl:when test="successors">
                <xsl:element name="successors" >
                    <xsl:for-each select="successors/successor">
                
                        <xsl:element name="successor">
                            <xsl:attribute name="id">
                                <xsl:value-of select="uri"/>
                            </xsl:attribute>         
                            
                            <xsl:if test="label != ''">
                                <xsl:element name="label" >
                                    <xsl:attribute name="xml:lang">
                                        <xsl:value-of select="label/@language"/>
                                    </xsl:attribute>
                                    <xsl:value-of select="label" />
                                </xsl:element>
                            </xsl:if>
                                              
                        </xsl:element>                
                    </xsl:for-each>
                </xsl:element>
            </xsl:when>
            <xsl:otherwise>
            </xsl:otherwise>
        </xsl:choose>
        
        
        <!--EnumerationValues-->
        <xsl:choose>
            <xsl:when test="collection">
                <xsl:element name="containeditems" >
                    <xsl:for-each select="collection/item">
                
                        <xsl:element name="value" >
                            
                            <xsl:choose>
                                <xsl:when test="uriname">
                                    <xsl:attribute name="id">
                                        <xsl:value-of select="uri"/>
                                    </xsl:attribute>         
                            
                                    <xsl:if test="thisversion != ''">
                                        <xsl:element name="thisversion" >
                                            <xsl:value-of select="thisversion/uri" />
                                        </xsl:element>
                                    </xsl:if>
        
                                    <xsl:if test="latestversion != ''">
                                        <xsl:element name="latestversion" >
                                            <xsl:value-of select="latestversion/uri" />
                                        </xsl:element>
                                    </xsl:if>
          
                                    <xsl:if test="previousversions != ''">
                                        <xsl:element name="previousversions" >
                                            <xsl:for-each select="previousversions/version">
                                                <xsl:element name="version">
                                                    <xsl:value-of select="uri"/>         
                                                </xsl:element>                
                                            </xsl:for-each>
                                        </xsl:element>
                                    </xsl:if>
                              
                              
                                    <xsl:if test="label != ''">
                                        <xsl:element name="label" >
                                            <xsl:attribute name="xml:lang">
                                                <xsl:value-of select="label/@language"/>
                                            </xsl:attribute>
                                            <xsl:value-of select="uriname" />
                                        </xsl:element>
                                    </xsl:if>
                    
                                    <xsl:if test="definition != ''">
                                        <xsl:variable name="tmp" select="definition"/>
                                
                                        <xsl:element name="definition" >
                                            <xsl:attribute name="xml:lang">
                                                <xsl:value-of select="definition/@language"/>
                                            </xsl:attribute>
                                            <xsl:value-of select="$tmp" />
                                        </xsl:element>
                                    </xsl:if>
                    
                                    <xsl:if test="description != ''">
                                        <xsl:variable name="tmp" select="description"/>
                                
                                        <xsl:element name="description" >
                                            <xsl:attribute name="xml:lang">
                                                <xsl:value-of select="description/@language"/>
                                            </xsl:attribute>
                                            <xsl:value-of select="$tmp" />
                                        </xsl:element>         
                                    </xsl:if>
                            
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:attribute name="id">
                                        <xsl:value-of select="uristandard"/>
                                    </xsl:attribute>
                                </xsl:otherwise>
                            </xsl:choose>
                            
                            <xsl:element name="itemclass" >
                                <xsl:attribute name="uriname">
                                    <xsl:value-of select="itemclass/uriname"/>
                                </xsl:attribute>
            
                                <xsl:element name="label" >
                                    <xsl:attribute name="xml:lang">
                                        <xsl:value-of select="itemclass/label/@language"/>
                                    </xsl:attribute>
                                    <xsl:value-of select="itemclass/label" />
                                </xsl:element>
                            </xsl:element>
                            
                            <xsl:if test="uriname">
                                <xsl:element name="status" >
                                    <xsl:attribute name="id">
                                        <xsl:value-of select="status/baseuri"/>
                                        <xsl:text>/</xsl:text>
                                        <xsl:value-of select="status/uriname"/>
                                    </xsl:attribute>
            
                                    <xsl:element name="label" >
                                        <xsl:attribute name="xml:lang">
                                            <xsl:value-of select="status/label/@language"/>
                                        </xsl:attribute>
                                        <xsl:value-of select="status/label" />
                                    </xsl:element>
                                </xsl:element>
                            </xsl:if>
                            
                            <xsl:if test="customattributes/customattribute[name/text()='GovernanceLevel'] != ''">            
                                <xsl:if test="customattributes/customattribute[name/text()='GovernanceLevel']/values != ''">            
                                    <xsl:for-each select="customattributes/customattribute[name/text()='GovernanceLevel']/values/value">                   
                                        <xsl:variable name="governance_uriname" select="uriname"/>
                                        <xsl:variable name="governance_url" select="concat($registryUri,'/governance-level/', $governance_uriname)"/>
                                        <xsl:if test="$governance_uriname != ''">                    
                                            <xsl:element name="governance-level" >
                                                <xsl:attribute name="id">
                                                    <xsl:value-of select="$governance_url"/>
                                                </xsl:attribute>
                                                <xsl:element name="label" >
                                                    <xsl:value-of select="$governance_uriname"/>
                                                </xsl:element>    
                                            </xsl:element>  
                                        </xsl:if>              
                                    </xsl:for-each>
                                </xsl:if>
                            </xsl:if>
                            
                            <xsl:if test="codelist != ''">
                                <xsl:element name="codelist" >
                                    <xsl:attribute name="id">
                                        <xsl:value-of select="codelist/uri"/>
                                    </xsl:attribute>
                                
                                    <label>
                                        <xsl:attribute name="xml:lang">
                                            <xsl:value-of select="codelist/label/@language"/>
                                        </xsl:attribute>
                                        <xsl:value-of select="codelist/label"/>
                                    </label>
                                </xsl:element>  
                            </xsl:if>
                            
                            <xsl:if test="customattributes/customattribute[name/text()='Theme'] != ''">            
                                <xsl:if test="customattributes/customattribute[name/text()='Theme']/values != ''">            
                                    <xsl:for-each select="customattributes/customattribute[name/text()='Theme']/values/value">                   
                                        <xsl:variable name="themeUri" select="uri"></xsl:variable>
                                        <xsl:element name="theme">
                                            <xsl:attribute name="id">
                                                <xsl:value-of select="$themeUri"/>
                                            </xsl:attribute>
                                            
                                            <xsl:if test="label">
                                                <xsl:variable name="themeLabel" select="label"></xsl:variable>
                                                <xsl:variable name="themeLabelLanguage" select="label/@language"></xsl:variable>
                                                <label>
                                                    <xsl:attribute name="xml:lang">
                                                        <xsl:value-of select="$themeLabelLanguage"/>
                                                    </xsl:attribute>
                                                    <xsl:value-of select="$themeLabel"/>
                                                </label>
                                            </xsl:if>
                                        </xsl:element>              
                                    </xsl:for-each>
                                </xsl:if>
                            </xsl:if>
        
                            <xsl:if test="customattributes/customattribute[name/text()='ApplicationSchema'] != ''">            
                                <xsl:if test="customattributes/customattribute[name/text()='ApplicationSchema']/values != ''">            
                                    <xsl:for-each select="customattributes/customattribute[name/text()='ApplicationSchema']/values/value">                   
                                        <xsl:variable name="applicationschemaUri" select="uri"></xsl:variable>
                                        <xsl:element name="applicationschema">
                                            <xsl:attribute name="id">
                                                <xsl:value-of select="$applicationschemaUri"/>
                                            </xsl:attribute>
                                            
                                            <xsl:if test="label">
                                                <xsl:variable name="applicationschemaLabel" select="label"></xsl:variable>
                                                <xsl:variable name="applicationschemaLabelLanguage" select="label/@language"></xsl:variable>
                                                <label>
                                                    <xsl:attribute name="xml:lang">
                                                        <xsl:value-of select="$applicationschemaLabelLanguage"/>
                                                    </xsl:attribute>
                                                    <xsl:value-of select="$applicationschemaLabel"/>
                                                </label>
                                            </xsl:if>
                                        </xsl:element>              
                                    </xsl:for-each>
                                </xsl:if>
                            </xsl:if> 
                            
                            <xsl:element name="register" >
                                <xsl:attribute name="id">
                                    <xsl:value-of select="register/uri"/>
                                </xsl:attribute>            
                                <label>
                                    <xsl:attribute name="xml:lang">
                                        <xsl:value-of select="register/label/@language"/>
                                    </xsl:attribute>
                                    <xsl:value-of select="register/label"/>
                                </label>
                                
                                <xsl:element name="registry" >
                                    <xsl:attribute name="id">
                                        <xsl:value-of select="register/registry/uri"/>
                                    </xsl:attribute>            
                                    <label>
                                        <xsl:attribute name="xml:lang">
                                            <xsl:value-of select="register/registry/label/@language"/>
                                        </xsl:attribute>
                                        <xsl:value-of select="register/registry/label"/>
                                    </label>
                                </xsl:element>
                            </xsl:element> 
                            
                            <xsl:choose>
                                <xsl:when test="parents">
                                    <xsl:element name="parents" >
                                        <xsl:for-each select="parents/parent">
                
                                            <xsl:element name="parent">
                                                <xsl:attribute name="id">
                                                    <xsl:value-of select="uri"/>
                                                </xsl:attribute>         
                            
                                                <xsl:if test="label != ''">
                                                    <xsl:element name="label" >
                                                        <xsl:attribute name="xml:lang">
                                                            <xsl:value-of select="label/@language"/>
                                                        </xsl:attribute>
                                                        <xsl:value-of select="label" />
                                                    </xsl:element>
                                                </xsl:if>
                                              
                                            </xsl:element>                
                                        </xsl:for-each>
                                    </xsl:element>
                                </xsl:when>
                                <xsl:otherwise>
                                </xsl:otherwise>
                            </xsl:choose>
                            
                            <xsl:choose>
                                <xsl:when test="predecessors">
                                    <xsl:element name="predecessors" >
                                        <xsl:for-each select="predecessors/predecessor">
                
                                            <xsl:element name="predecessor">
                                                <xsl:attribute name="id">
                                                    <xsl:value-of select="uri"/>
                                                </xsl:attribute>         
                            
                                                <xsl:if test="label != ''">
                                                    <xsl:element name="label" >
                                                        <xsl:attribute name="xml:lang">
                                                            <xsl:value-of select="label/@language"/>
                                                        </xsl:attribute>
                                                        <xsl:value-of select="label" />
                                                    </xsl:element>
                                                </xsl:if>
                                              
                                            </xsl:element>                
                                        </xsl:for-each>
                                    </xsl:element>
                                </xsl:when>
                                <xsl:otherwise>
                                </xsl:otherwise>
                            </xsl:choose>
        
                            <xsl:choose>
                                <xsl:when test="successors">
                                    <xsl:element name="successors" >
                                        <xsl:for-each select="successors/successor">
                
                                            <xsl:element name="successor">
                                                <xsl:attribute name="id">
                                                    <xsl:value-of select="uri"/>
                                                </xsl:attribute>         
                            
                                                <xsl:if test="label != ''">
                                                    <xsl:element name="label" >
                                                        <xsl:attribute name="xml:lang">
                                                            <xsl:value-of select="label/@language"/>
                                                        </xsl:attribute>
                                                        <xsl:value-of select="label" />
                                                    </xsl:element>
                                                </xsl:if>
                                              
                                            </xsl:element>                
                                        </xsl:for-each>
                                    </xsl:element>
                                </xsl:when>
                                <xsl:otherwise>
                                </xsl:otherwise>
                            </xsl:choose>
                                              
                        </xsl:element>                
                    </xsl:for-each>
                </xsl:element>
            </xsl:when>
            <xsl:otherwise>
            </xsl:otherwise>
        </xsl:choose>  
              
              
        <!-- Enumerations-->
        <xsl:choose>
            <xsl:when test="children">
                <xsl:element name="containeditems" >
                    <xsl:for-each select="children/item">
                
                        <xsl:element name="enumeration" >
                            <xsl:attribute name="id">
                                <xsl:value-of select="uri"/>
                            </xsl:attribute>
                            
                            <xsl:element name="version">
                                <xsl:value-of select="versionnumber"/>
                            </xsl:element>  
                            
                            <xsl:if test="label != ''">
                                <xsl:element name="label" >
                                    <xsl:attribute name="xml:lang">
                                        <xsl:value-of select="label/@language"/>
                                    </xsl:attribute>
                                    <xsl:value-of select="label" />
                                </xsl:element>
                            </xsl:if>
                    
                            <xsl:if test="definition != ''">
                                <xsl:variable name="tmp" select="definition"/>

                                <xsl:element name="definition" >
                                    <xsl:attribute name="xml:lang">
                                        <xsl:value-of select="definition/@language"/>
                                    </xsl:attribute>
                                    <xsl:value-of select="$tmp" />
                                </xsl:element>
                            </xsl:if>

                            <xsl:if test="description != ''">
                                <xsl:variable name="tmp" select="description"/>

                                <xsl:element name="description" >
                                    <xsl:attribute name="xml:lang">
                                        <xsl:value-of select="description/@language"/>
                                    </xsl:attribute>
                                    <xsl:value-of select="$tmp" />
                                </xsl:element>
                            </xsl:if>
                    
                            <xsl:element name="itemclass" >
                                <xsl:attribute name="uriname">
                                    <xsl:value-of select="itemclass/uriname"/>
                                </xsl:attribute>
            
                                <xsl:element name="label" >
                                    <xsl:attribute name="xml:lang">
                                        <xsl:value-of select="itemclass/label/@language"/>
                                    </xsl:attribute>
                                    <xsl:value-of select="itemclass/label" />
                                </xsl:element>
                            </xsl:element>
                    
                            <xsl:element name="status" >
                                <xsl:attribute name="id">
                                    <xsl:value-of select="status/baseuri"/>
                                    <xsl:text>/</xsl:text>
                                    <xsl:value-of select="status/uriname"/>
                                </xsl:attribute>
            
                                <xsl:element name="label" >
                                    <xsl:attribute name="xml:lang">
                                        <xsl:value-of select="status/label/@language"/>
                                    </xsl:attribute>
                                    <xsl:value-of select="status/label" />
                                </xsl:element>
                            </xsl:element> 
        
                            <xsl:if test="customattributes/customattribute[name/text()='Extensibility'] != ''">            
                                <xsl:if test="customattributes/customattribute[name/text()='Extensibility']/values != ''">            
                                    <xsl:for-each select="customattributes/customattribute[name/text()='Extensibility']/values/value">                   
                                        <xsl:variable name="extensibility_label" select="uriname"></xsl:variable>
                                        <xsl:variable name="extensibility_url" select="concat($registryUri,'/extensibility/', $extensibility_label)"/>
                                        <xsl:if test="$extensibility_label != ''">
                                            <xsl:element name="extensibility" >
                                                <xsl:attribute name="id">
                                                    <xsl:value-of select="$extensibility_url"/>
                                                </xsl:attribute>
            
                                                <xsl:element name="uriname" >
                                                    <xsl:value-of select="$extensibility_label" />
                                                </xsl:element>
                                            </xsl:element>
                                        </xsl:if>            
                                    </xsl:for-each>
                                </xsl:if>
                            </xsl:if>
                            
                            <xsl:if test="customattributes/customattribute[name/text()='GovernanceLevel'] != ''">            
                                <xsl:if test="customattributes/customattribute[name/text()='GovernanceLevel']/values != ''">            
                                    <xsl:for-each select="customattributes/customattribute[name/text()='GovernanceLevel']/values/value">                   
                                        <xsl:variable name="governance_uriname" select="uriname"/>
                                        <xsl:variable name="governance_url" select="concat($registryUri,'/governance-level/', $governance_uriname)"/>
                                        <xsl:if test="$governance_uriname != ''">                    
                                            <xsl:element name="governance-level" >
                                                <xsl:attribute name="id">
                                                    <xsl:value-of select="$governance_url"/>
                                                </xsl:attribute>
                                                <xsl:element name="label" >
                                                    <xsl:value-of select="$governance_uriname"/>
                                                </xsl:element>    
                                            </xsl:element>  
                                        </xsl:if>              
                                    </xsl:for-each>
                                </xsl:if>
                            </xsl:if>
                            
                            <xsl:choose>
                                <xsl:when test="parents">
                                    <xsl:element name="parents" >
                                        <xsl:for-each select="parents/parent">
                
                                            <xsl:element name="parent">
                                                <xsl:attribute name="id">
                                                    <xsl:value-of select="uri"/>
                                                </xsl:attribute>         
                            
                                                <xsl:if test="label != ''">
                                                    <xsl:element name="label" >
                                                        <xsl:attribute name="xml:lang">
                                                            <xsl:value-of select="label/@language"/>
                                                        </xsl:attribute>
                                                        <xsl:value-of select="label" />
                                                    </xsl:element>
                                                </xsl:if>
                                              
                                            </xsl:element>                
                                        </xsl:for-each>
                                    </xsl:element>
                                </xsl:when>
                                <xsl:otherwise>
                                </xsl:otherwise>
                            </xsl:choose>
                            
                            <xsl:element name="register" >
                                <xsl:attribute name="id">
                                    <xsl:value-of select="register/uri"/>
                                </xsl:attribute>
            
                                <label>
                                    <xsl:attribute name="xml:lang">
                                        <xsl:value-of select="register/label/@language"/>
                                    </xsl:attribute>
                                    <xsl:value-of select="register/label"/>
                                </label>
                                
                                <xsl:element name="registry" >
                                    <xsl:attribute name="id">
                                        <xsl:value-of select="register/registry/uri"/>
                                    </xsl:attribute>
            
                                    <label>
                                        <xsl:attribute name="xml:lang">
                                            <xsl:value-of select="register/registry/label/@language"/>
                                        </xsl:attribute>
                                        <xsl:value-of select="register/registry/label"/>
                                    </label>
                                </xsl:element>
                            </xsl:element>   
                            
                            <xsl:if test="customattributes/customattribute[name/text()='Theme'] != ''">            
                                <xsl:if test="customattributes/customattribute[name/text()='Theme']/values != ''">            
                                    <xsl:for-each select="customattributes/customattribute[name/text()='Theme']/values/value">                   
                                        <xsl:variable name="themeUri" select="uri"></xsl:variable>
                                        <xsl:element name="theme">
                                            <xsl:attribute name="id">
                                                <xsl:value-of select="$themeUri"/>
                                            </xsl:attribute>
                                            
                                            <xsl:if test="label">
                                                <xsl:variable name="themeLabel" select="label"></xsl:variable>
                                                <xsl:variable name="themeLabelLanguage" select="label/@language"></xsl:variable>
                                                <label>
                                                    <xsl:attribute name="xml:lang">
                                                        <xsl:value-of select="$themeLabelLanguage"/>
                                                    </xsl:attribute>
                                                    <xsl:value-of select="$themeLabel"/>
                                                </label>
                                            </xsl:if>
                                        </xsl:element>              
                                    </xsl:for-each>
                                </xsl:if>
                            </xsl:if>
        
                            <xsl:if test="customattributes/customattribute[name/text()='ApplicationSchema'] != ''">            
                                <xsl:if test="customattributes/customattribute[name/text()='ApplicationSchema']/values != ''">            
                                    <xsl:for-each select="customattributes/customattribute[name/text()='ApplicationSchema']/values/value">                   
                                        <xsl:variable name="applicationschemaUri" select="uri"></xsl:variable>
                                        <xsl:element name="applicationschema">
                                            <xsl:attribute name="id">
                                                <xsl:value-of select="$applicationschemaUri"/>
                                            </xsl:attribute>
                                            
                                            <xsl:if test="label">
                                                <xsl:variable name="applicationschemaLabel" select="label"></xsl:variable>
                                                <xsl:variable name="applicationschemaLabelLanguage" select="label/@language"></xsl:variable>
                                                <label>
                                                    <xsl:attribute name="xml:lang">
                                                        <xsl:value-of select="$applicationschemaLabelLanguage"/>
                                                    </xsl:attribute>
                                                    <xsl:value-of select="$applicationschemaLabel"/>
                                                </label>
                                            </xsl:if>
                                        </xsl:element>              
                                    </xsl:for-each>
                                </xsl:if>
                            </xsl:if> 
                                                                
                            <xsl:choose>
                                <xsl:when test="collection">
                                    <xsl:element name="containeditems" >
                                        <xsl:for-each select="collection/item">
                
                                            <xsl:element name="value" >
                                                <xsl:attribute name="id">
                                                    <xsl:value-of select="uri"/>
                                                </xsl:attribute>         
                            
                                                <xsl:if test="thisversion != ''">
                                                    <xsl:element name="thisversion" >
                                                        <xsl:value-of select="thisversion/uri" />
                                                    </xsl:element>
                                                </xsl:if>
        
                                                <xsl:if test="latestversion != ''">
                                                    <xsl:element name="latestversion" >
                                                        <xsl:value-of select="latestversion/uri" />
                                                    </xsl:element>
                                                </xsl:if>
          
                                                <xsl:if test="previousversions != ''">
                                                    <xsl:element name="previousversions" >
                                                        <xsl:for-each select="previousversions/version">
                                                            <xsl:element name="version">
                                                                <xsl:value-of select="uri"/>         
                                                            </xsl:element>                
                                                        </xsl:for-each>
                                                    </xsl:element>
                                                </xsl:if>
                              
                              
                                                <xsl:if test="label != ''">
                                                    <xsl:element name="label" >
                                                        <xsl:attribute name="xml:lang">
                                                            <xsl:value-of select="label/@language"/>
                                                        </xsl:attribute>
                                                        <xsl:value-of select="label" />
                                                    </xsl:element>
                                                </xsl:if>
                    
                                                <xsl:if test="definition != ''">
                                                    <xsl:variable name="tmp" select="definition"/>
                                
                                                    <xsl:element name="definition" >
                                                        <xsl:attribute name="xml:lang">
                                                            <xsl:value-of select="definition/@language"/>
                                                        </xsl:attribute>
                                                        <xsl:value-of select="$tmp" />
                                                    </xsl:element>
                                                </xsl:if>
                    
                                                <xsl:if test="description != ''">
                                                    <xsl:variable name="tmp" select="description"/>
                                
                                                    <xsl:element name="description" >
                                                        <xsl:attribute name="xml:lang">
                                                            <xsl:value-of select="description/@language"/>
                                                        </xsl:attribute>
                                                        <xsl:value-of select="$tmp" />
                                                    </xsl:element>         
                                                </xsl:if>
                            
                                                <xsl:if test="customattributes/customattribute[name/text()='GovernanceLevel'] != ''">            
                                                    <xsl:if test="customattributes/customattribute[name/text()='GovernanceLevel']/values != ''">            
                                                        <xsl:for-each select="customattributes/customattribute[name/text()='GovernanceLevel']/values/value">                   
                                                            <xsl:variable name="governance_uriname" select="uriname"/>
                                                            <xsl:variable name="governance_url" select="concat($registryUri,'/governance-level/', $governance_uriname)"/>
                                                            <xsl:if test="$governance_uriname != ''">                    
                                                                <xsl:element name="governance-level" >
                                                                    <xsl:attribute name="id">
                                                                        <xsl:value-of select="$governance_url"/>
                                                                    </xsl:attribute>
                                                                    <xsl:element name="label" >
                                                                        <xsl:value-of select="$governance_uriname"/>
                                                                    </xsl:element>    
                                                                </xsl:element>  
                                                            </xsl:if>              
                                                        </xsl:for-each>
                                                    </xsl:if>
                                                </xsl:if>
                            
                                                <xsl:if test="codelist != ''">
                                                    <xsl:element name="codelist" >
                                                        <xsl:attribute name="id">
                                                            <xsl:value-of select="codelist/uri"/>
                                                        </xsl:attribute>
                                
                                                        <label>
                                                            <xsl:attribute name="xml:lang">
                                                                <xsl:value-of select="codelist/label/language"/>
                                                            </xsl:attribute>
                                                            <xsl:value-of select="codelist/label"/>
                                                        </label>
                                                    </xsl:element>  
                                                </xsl:if>
                            
                                                <xsl:if test="customattributes/customattribute[name/text()='Theme'] != ''">            
                                                    <xsl:if test="customattributes/customattribute[name/text()='Theme']/values != ''">            
                                                        <xsl:for-each select="customattributes/customattribute[name/text()='Theme']/values/value">                   
                                                            <xsl:variable name="themeUri" select="uri"></xsl:variable>
                                                            <xsl:element name="theme">
                                                                <xsl:attribute name="id">
                                                                    <xsl:value-of select="$themeUri"/>
                                                                </xsl:attribute>
                                                                
                                                                <xsl:if test="label">
                                                                    <xsl:variable name="themeLabel" select="label"></xsl:variable>
                                                                    <xsl:variable name="themeLabelLanguage" select="label/@language"></xsl:variable>
                                                                    <label>
                                                                        <xsl:attribute name="xml:lang">
                                                                            <xsl:value-of select="$themeLabelLanguage"/>
                                                                        </xsl:attribute>
                                                                        <xsl:value-of select="$themeLabel"/>
                                                                    </label>
                                                                </xsl:if>
                                                            </xsl:element>              
                                                        </xsl:for-each>
                                                    </xsl:if>
                                                </xsl:if>
        
                                                <xsl:if test="customattributes/customattribute[name/text()='ApplicationSchema'] != ''">            
                                                    <xsl:if test="customattributes/customattribute[name/text()='ApplicationSchema']/values != ''">            
                                                        <xsl:for-each select="customattributes/customattribute[name/text()='ApplicationSchema']/values/value">                   
                                                            <xsl:variable name="applicationschemaUri" select="uri"></xsl:variable>
                                                            <xsl:element name="applicationschema">
                                                                <xsl:attribute name="id">
                                                                    <xsl:value-of select="$applicationschemaUri"/>
                                                                </xsl:attribute>
                                                                
                                                                <xsl:if test="label">
                                                                    <xsl:variable name="applicationschemaLabel" select="label"></xsl:variable>
                                                                    <xsl:variable name="applicationschemaLabelLanguage" select="label/@language"></xsl:variable>
                                                                    <label>
                                                                        <xsl:attribute name="xml:lang">
                                                                            <xsl:value-of select="$applicationschemaLabelLanguage"/>
                                                                        </xsl:attribute>
                                                                        <xsl:value-of select="$applicationschemaLabel"/>
                                                                    </label>
                                                                </xsl:if>
                                                            </xsl:element>              
                                                        </xsl:for-each>
                                                    </xsl:if>
                                                </xsl:if> 
                            
                                                <xsl:element name="itemclass" >
                                                    <xsl:attribute name="uriname">
                                                        <xsl:value-of select="itemclass/uriname"/>
                                                    </xsl:attribute>
            
                                                    <xsl:element name="label" >
                                                        <xsl:attribute name="xml:lang">
                                                            <xsl:value-of select="itemclass/label/@language"/>
                                                        </xsl:attribute>
                                                        <xsl:value-of select="itemclass/label" />
                                                    </xsl:element>
                                                </xsl:element>
                            
                                                <xsl:element name="status" >
                                                    <xsl:attribute name="id">
                                                        <xsl:value-of select="status/baseuri"/>
                                                        <xsl:text>/</xsl:text>
                                                        <xsl:value-of select="status/uriname"/>
                                                    </xsl:attribute>
            
                                                    <xsl:element name="label" >
                                                        <xsl:attribute name="xml:lang">
                                                            <xsl:value-of select="status/label/@language"/>
                                                        </xsl:attribute>
                                                        <xsl:value-of select="status/label" />
                                                    </xsl:element>
                                                </xsl:element>
                            
                                                <xsl:element name="register" >
                                                    <xsl:attribute name="id">
                                                        <xsl:value-of select="register/uri"/>
                                                    </xsl:attribute>            
                                                    <label>
                                                        <xsl:attribute name="xml:lang">
                                                            <xsl:value-of select="register/label/@language"/>
                                                        </xsl:attribute>
                                                        <xsl:value-of select="register/label"/>
                                                    </label>
                                
                                                    <xsl:element name="registry" >
                                                        <xsl:attribute name="id">
                                                            <xsl:value-of select="register/registry/uri"/>
                                                        </xsl:attribute>            
                                                        <label>
                                                            <xsl:attribute name="xml:lang">
                                                                <xsl:value-of select="register/registry/label/@language"/>
                                                            </xsl:attribute>
                                                            <xsl:value-of select="register/registry/label"/>
                                                        </label>
                                                    </xsl:element>
                                                </xsl:element> 
                            
                                                <xsl:choose>
                                                    <xsl:when test="parents">
                                                        <xsl:element name="parents" >
                                                            <xsl:for-each select="parents/parent">
                
                                                                <xsl:element name="parent">
                                                                    <xsl:attribute name="id">
                                                                        <xsl:value-of select="uri"/>
                                                                    </xsl:attribute>         
                            
                                                                    <xsl:if test="label != ''">
                                                                        <xsl:element name="label" >
                                                                            <xsl:attribute name="xml:lang">
                                                                                <xsl:value-of select="label/@language"/>
                                                                            </xsl:attribute>
                                                                            <xsl:value-of select="label" />
                                                                        </xsl:element>
                                                                    </xsl:if>
                                              
                                                                </xsl:element>                
                                                            </xsl:for-each>
                                                        </xsl:element>
                                                    </xsl:when>
                                                    <xsl:otherwise>
                                                    </xsl:otherwise>
                                                </xsl:choose>

                                                <xsl:choose>
                                                    <xsl:when test="predecessors">
                                                        <xsl:element name="predecessors" >
                                                            <xsl:for-each select="predecessors/predecessor">
                
                                                                <xsl:element name="predecessor">
                                                                    <xsl:attribute name="id">
                                                                        <xsl:value-of select="uri"/>
                                                                    </xsl:attribute>         
                            
                                                                    <xsl:if test="label != ''">
                                                                        <xsl:element name="label" >
                                                                            <xsl:attribute name="xml:lang">
                                                                                <xsl:value-of select="label/@language"/>
                                                                            </xsl:attribute>
                                                                            <xsl:value-of select="label" />
                                                                        </xsl:element>
                                                                    </xsl:if>
                                              
                                                                </xsl:element>                
                                                            </xsl:for-each>
                                                        </xsl:element>
                                                    </xsl:when>
                                                    <xsl:otherwise>
                                                    </xsl:otherwise>
                                                </xsl:choose>
        
                                                <xsl:choose>
                                                    <xsl:when test="successors">
                                                        <xsl:element name="successors" >
                                                            <xsl:for-each select="successors/successor">
                
                                                                <xsl:element name="successor">
                                                                    <xsl:attribute name="id">
                                                                        <xsl:value-of select="uri"/>
                                                                    </xsl:attribute>         
                            
                                                                    <xsl:if test="label != ''">
                                                                        <xsl:element name="label" >
                                                                            <xsl:attribute name="xml:lang">
                                                                                <xsl:value-of select="label/@language"/>
                                                                            </xsl:attribute>
                                                                            <xsl:value-of select="label" />
                                                                        </xsl:element>
                                                                    </xsl:if>
                                              
                                                                </xsl:element>                
                                                            </xsl:for-each>
                                                        </xsl:element>
                                                    </xsl:when>
                                                    <xsl:otherwise>
                                                    </xsl:otherwise>
                                                </xsl:choose>
                                              
                                            </xsl:element>                
                                        </xsl:for-each>
                                    </xsl:element>
                                </xsl:when>
                                <xsl:otherwise>
                                </xsl:otherwise>
                            </xsl:choose>  
                                
                        </xsl:element>
                    </xsl:for-each>
                </xsl:element>
        
            </xsl:when>
            <xsl:otherwise>
            </xsl:otherwise>
        </xsl:choose>
              
                          
    </xsl:template>
</xsl:stylesheet>