<!--  Copyright 2010,2015 EUROPEAN UNION
  Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
  the European Commission - subsequent versions of the EUPL (the "Licence");
  You may not use this work except in compliance with the Licence.
  You may obtain a copy of the Licence at:
 
  http://ec.europa.eu/idabc/eupl
 
  Unless required by applicable law or agreed to in writing, software
  distributed under the Licence is distributed on an "AS IS" basis,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the Licence for the specific language governing permissions and
  limitations under the Licence.
 
  Date: 15/07/2016
  Authors: European Commission, Joint Research Centre
  Daniele Francioli, Emanuela Epure
  inspire-registry-dev@jrc.ec.europa.eu
 
This work was supported by the EU  Interoperability Solutions for
European Public Administrations Programme (http://ec.europa.eu/isa)
through Action 1.17: Re-usable INSPIRE Reference Platform 
(http://ec.europa.eu/isa/actions/01-trusted-information-exchange/1-17action_en.htm).-->

<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xs="http://www.w3.org/2001/XMLSchema"> 
    
    <!--Output settings-->
    <xsl:output method="text" encoding="UTF-8" media-type="text/plain" omit-xml-declaration="yes"/>

    <!--Variables & parameters declaration-->
    <xsl:param name="base_db_export_path" />
    
    <xsl:template match="/">
        {
        "country":{
        <xsl:apply-templates select="item"/>
        }
        }
    </xsl:template>
    
    <xsl:template match="item">
        
        "id":"<xsl:value-of select="uri"/>"
        <xsl:if test="thisversion != ''">
            ,"thisversion":"<xsl:value-of select="thisversion/uri" />"
        </xsl:if>
        <xsl:if test="latestversion != ''">
            ,"latestversion":"<xsl:value-of select="latestversion/uri" />"
        </xsl:if>
        <xsl:choose>
            <xsl:when test="previousversions">
                ,"historyversion":[
                <xsl:for-each select="previousversions/version">   
                    {
                    "version":"<xsl:value-of select="uri"/>"
                    }                
                    <xsl:choose>
                        <xsl:when test="following-sibling::version">
                            , 
                        </xsl:when>
                    </xsl:choose>
                </xsl:for-each>
                ]
            </xsl:when>
            <xsl:otherwise>
            </xsl:otherwise>
        </xsl:choose>
        <xsl:variable name="language" select="language/isocode"></xsl:variable>   
        ,"language":"<xsl:value-of select="$language"/>",
        <xsl:if test="label != ''">
            <xsl:variable name="tmp" select="label"/>
            <xsl:variable name="newtext" select="replace($tmp,'&quot;','')"/>
            "label":{
             "lang":"<xsl:value-of select="label/@language"/>",
             "text":"<xsl:value-of select="$newtext" />"
            },
        </xsl:if>   
        <xsl:if test="definition != ''">
            <xsl:variable name="tmp" select="definition"/>
            <xsl:variable name="newtext" select="replace($tmp,'&quot;','')"/>
            "definition":{
             "lang":"<xsl:value-of select="definition/@language"/>",
             "text":"<xsl:value-of select="$newtext" />"
            },
        </xsl:if>
        <xsl:if test="description != ''">
            <xsl:variable name="tmp" select="description"/>
            <xsl:variable name="newtext" select="replace($tmp,'&quot;','')"/>
            "description":{
             "lang":"<xsl:value-of select="description/@language"/>",
             "text":"<xsl:value-of select="$newtext" />"
            },
        </xsl:if>
        "itemclass":{
          "id":"<xsl:value-of select="itemclass/uriname"/>",
          "label":{
            "lang":"<xsl:value-of select="itemclass/label/@language"/>",
            "text":"<xsl:value-of select="itemclass/label" />"
        }
        }
        ,"status":{
        "id":" <xsl:value-of select="status/baseuri"/>
        <xsl:text>/</xsl:text>
        <xsl:value-of select="status/uriname"/>",
        "label":{
            "lang":"<xsl:value-of select="status/label/@language"/>",
            "text":"<xsl:value-of select="status/label" />"
        }
        }
        <xsl:choose>
            <xsl:when test="parents">
                ,"parents":[
                <xsl:for-each select="parents/parent">   
                    {
                    "parent":{
                     "id":"<xsl:value-of select="uri"/>",
                     <xsl:variable name="tmp" select="label"/>
                     <xsl:variable name="newtext" select="replace($tmp,'&quot;','')"/>
                     "label":{
                         "lang":"<xsl:value-of select="label/@language"/>",
                         "text":"<xsl:value-of select="$newtext" />"
                     }
                    }
                    }                
                    <xsl:choose>
                        <xsl:when test="following-sibling::predecessor">
                            , 
                        </xsl:when>
                    </xsl:choose>
                </xsl:for-each>
                ]
            </xsl:when>
            <xsl:otherwise>
            </xsl:otherwise>
        </xsl:choose>
        ,"register":{
            "id":"<xsl:value-of select="register/uri"/>",
            "label":{
               "lang":"<xsl:value-of select="register/label/@language"/>",
               "text":"<xsl:value-of select="register/label"/>"
        },
        "registry":{
          "id":"<xsl:value-of select="register/registry/uri"/>",
          "label":{
            "lang":"<xsl:value-of select="register/registry/label/@language"/>",
            "text":"<xsl:value-of select="register/registry/label"/>"
          }
        }
        }
        <xsl:if test="(customattributes/customattribute[name/text()='SourceTitle'] != '') and (customattributes/customattribute[name/text()='SourceLink'] != '')">
            <xsl:variable name="sourceTitle" select="customattributes/customattribute[name/text()='SourceTitle']/values/value/uriname"/>
            <xsl:variable name="sourceLink" select="customattributes/customattribute[name/text()='SourceLink']/values/value/uri"/>
            ,"source":{
                "title":"<xsl:value-of select="$sourceTitle"/>"
            <xsl:if test="$sourceLink != ''">
                ,"link":"<xsl:value-of select="$sourceLink"/>"
            </xsl:if>
            }	
        </xsl:if>
        <xsl:choose>
            <xsl:when test="predecessors">
                ,"predecessors":[
                <xsl:for-each select="predecessors/predecessor">   
                    {
                    "predecessor":{
                     "id":"<xsl:value-of select="uri"/>",
                     <xsl:variable name="tmp" select="label"/>
                     <xsl:variable name="newtext" select="replace($tmp,'&quot;','')"/>
                     "label":{
                        "lang":"<xsl:value-of select="label/@language"/>",
                        "text":"<xsl:value-of select="$newtext" />"
                     }
                    }
                    }                
                    <xsl:choose>
                        <xsl:when test="following-sibling::predecessor">
                            , 
                        </xsl:when>
                    </xsl:choose>
                </xsl:for-each>
                ]
            </xsl:when>
            <xsl:otherwise>
            </xsl:otherwise>
        </xsl:choose>
        
        <xsl:choose>
            <xsl:when test="successors">
                ,"successors":[
                <xsl:for-each select="successors/successor">   
                    {
                    "successor":{
                        "id":"<xsl:value-of select="uri"/>",
                        <xsl:variable name="tmp" select="label"/>
                        <xsl:variable name="newtext" select="replace($tmp,'&quot;','')"/>
                        "label":{
                         "lang":"<xsl:value-of select="label/@language"/>",
                         "text":"<xsl:value-of select="$newtext" />"
                        }
                    }
                    }                
                    <xsl:choose>
                        <xsl:when test="following-sibling::successor">
                            , 
                        </xsl:when>
                    </xsl:choose>
                </xsl:for-each>
                ]
            </xsl:when>
            <xsl:otherwise>
            </xsl:otherwise>
        </xsl:choose>
        
        <xsl:choose>
            <xsl:when test="collection">
                ,"containeditems":[
                <xsl:for-each select="collection/item">
                    {        
                    "value":{
                    <xsl:choose>
                        <xsl:when test="uriname">
                            "id":"<xsl:value-of select="uri"/>" 
                            <xsl:if test="thisversion != ''">
                                ,"thisversion":"<xsl:value-of select="thisversion/uri" />"
                            </xsl:if>
                            <xsl:if test="latestversion != ''">
                                ,"latestversion":"<xsl:value-of select="latestversion/uri" />"
                            </xsl:if>
                            <xsl:choose>
                                <xsl:when test="previousversions">
                                    ,"historyversion":[
                                    <xsl:for-each select="previousversions/version">   
                                        {
                                        "version":"<xsl:value-of select="uri"/>"
                                        }                
                                        <xsl:choose>
                                            <xsl:when test="following-sibling::version">
                                                , 
                                            </xsl:when>
                                        </xsl:choose>
                                    </xsl:for-each>
                                    ]
                                </xsl:when>
                                <xsl:otherwise>
                                </xsl:otherwise>
                            </xsl:choose>
                            ,
                            <xsl:variable name="tmp" select="label"/>
                            <xsl:variable name="newtext" select="replace($tmp,'&quot;','')"/>
                            "label":{
                                "lang":"<xsl:value-of select="label/@language"/>",
                                "text":"<xsl:value-of select="$newtext" />"
                            },
                            <xsl:if test="definition != ''">
                                <xsl:variable name="tmp" select="definition"/>
                                <xsl:variable name="newtext" select="replace($tmp,'&quot;','')"/>
                                "definition":{
                                 "lang":"<xsl:value-of select="definition/@language"/>",
                                 "text":"<xsl:value-of select="$newtext" />"
                                },
                            </xsl:if>
                            <xsl:if test="description != ''">
                                <xsl:variable name="tmp" select="description"/>
                                <xsl:variable name="newtext" select="replace($tmp,'&quot;','')"/>
                                "description":{
                                 "lang":"<xsl:value-of select="description/@language"/>",
                                 "text":"<xsl:value-of select="$newtext" />"
                                },
                            </xsl:if>
                        </xsl:when>
                        <xsl:otherwise>
                            "id":"<xsl:value-of select="uristandard"/>",
                        </xsl:otherwise>
                    </xsl:choose> 
                    "itemclass":{
                     "id":"<xsl:value-of select="itemclass/uriname"/>",
                     "label":{
                        "lang":"<xsl:value-of select="itemclass/label/@language"/>",
                        "text":"<xsl:value-of select="itemclass/label" />"
                     }
                    }
                    <xsl:if test="uriname">
                        ,"status":{
                        "id":"<xsl:value-of select="status/baseuri"/>
                        <xsl:text>/</xsl:text>
                        <xsl:value-of select="status/uriname"/>",
                        "label":{
                            "lang":"<xsl:value-of select="status/label/@language"/>",
                            "text":"<xsl:value-of select="status/label" />"
                        }
                        }
                    </xsl:if>
                    <xsl:choose>
                        <xsl:when test="parents">
                            ,"parents":[
                            <xsl:for-each select="parents/parent">   
                                {
                                "parent":{
                                 "id":"<xsl:value-of select="uri"/>",
                                 <xsl:variable name="tmp" select="label"/>
                                 <xsl:variable name="newtext" select="replace($tmp,'&quot;','')"/>
                                 "label":{
                                     "lang":"<xsl:value-of select="label/@language"/>",
                                     "text":"<xsl:value-of select="$newtext" />"
                                 }
                                }
                                }                
                                <xsl:choose>
                                    <xsl:when test="following-sibling::parent">
                                        , 
                                    </xsl:when>
                                </xsl:choose>
                            </xsl:for-each>
                            ]
                        </xsl:when>
                        <xsl:otherwise>
                        </xsl:otherwise>
                    </xsl:choose>
                    ,"register":{
                     "id":"<xsl:value-of select="register/uri"/>",
                     "label":{
                        "lang":"<xsl:value-of select="register/label/@language"/>",
                        "text":"<xsl:value-of select="register/label"/>"
                    },
                    "registry":{
                     "id":"<xsl:value-of select="register/registry/uri"/>",
                     "label":{
                         "lang":"<xsl:value-of select="register/registry/label/@language"/>",
                         "text":"<xsl:value-of select="register/registry/label"/>"
                     }
                    }
                    }
                    ,"country":{
                     "id":"<xsl:value-of select="codelist/uri"/>",
                     <xsl:variable name="tmp" select="codelist/label"/>
                     <xsl:variable name="newtext" select="replace($tmp,'&quot;','')"/>
                     "label":{
                        "lang":"<xsl:value-of select="label/@language"/>",
                        "text":"<xsl:value-of select="$newtext" />"
                     }
                    }
                    <xsl:if test="(customattributes/customattribute[name/text()='SourceTitle'] != '') and (customattributes/customattribute[name/text()='SourceLink'] != '')">
                        <xsl:variable name="sourceTitle" select="customattributes/customattribute[name/text()='SourceTitle']/values/value/uriname"/>
                        <xsl:variable name="sourceLink" select="customattributes/customattribute[name/text()='SourceLink']/values/value/uri"/>
                        ,"source":{
                            "title":"<xsl:value-of select="$sourceTitle"/>"
                        <xsl:if test="$sourceLink != ''">
                            ,"link":"<xsl:value-of select="$sourceLink"/>"
                        </xsl:if>
                        }	
                    </xsl:if>
                    <xsl:choose>
                        <xsl:when test="predecessors">
                            ,"predecessors":[
                            <xsl:for-each select="predecessors/predecessor">   
                                {
                                "predecessor":{
                                    "id":"<xsl:value-of select="uri"/>",
                                    <xsl:variable name="tmp" select="label"/>
                                    <xsl:variable name="newtext" select="replace($tmp,'&quot;','')"/>
                                    "label":{
                                     "lang":"<xsl:value-of select="label/@language"/>",
                                     "text":"<xsl:value-of select="$newtext" />"
                                    }
                                }
                                }                
                                <xsl:choose>
                                    <xsl:when test="following-sibling::predecessor">
                                        , 
                                    </xsl:when>
                                </xsl:choose>
                            </xsl:for-each>
                            ]
                        </xsl:when>
                        <xsl:otherwise>
                        </xsl:otherwise>
                    </xsl:choose>
                    
                    <xsl:choose>
                        <xsl:when test="successors">
                            ,"successors":[
                            <xsl:for-each select="successors/successor">   
                                {
                                "successor":{
                                 "id":"<xsl:value-of select="uri"/>",
                                 <xsl:variable name="tmp" select="label"/>
                                 <xsl:variable name="newtext" select="replace($tmp,'&quot;','')"/>
                                 "label":{
                                    "lang":"<xsl:value-of select="label/@language"/>",
                                    "text":"<xsl:value-of select="$newtext" />"
                                 }
                                }
                                }                
                                <xsl:choose>
                                    <xsl:when test="following-sibling::successor">
                                        , 
                                    </xsl:when>
                                </xsl:choose>
                            </xsl:for-each>
                            ]
                        </xsl:when>
                        <xsl:otherwise>
                        </xsl:otherwise>
                    </xsl:choose>
                    }
                    }
                    <xsl:choose>
                        <xsl:when test="following-sibling::item">
                            , 
                        </xsl:when>
                    </xsl:choose>
                </xsl:for-each>
                ]            
            </xsl:when>
            <xsl:otherwise>
            </xsl:otherwise>
        </xsl:choose>      
        
    </xsl:template>
</xsl:stylesheet>