<?xml version="1.0" encoding="ISO-8859-1"?>

<!--  Copyright 2010,2015 EUROPEAN UNION
  Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
  the European Commission - subsequent versions of the EUPL (the "Licence");
  You may not use this work except in compliance with the Licence.
  You may obtain a copy of the Licence at:
 
  http://ec.europa.eu/idabc/eupl
 
  Unless required by applicable law or agreed to in writing, software
  distributed under the Licence is distributed on an "AS IS" basis,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the Licence for the specific language governing permissions and
  limitations under the Licence.
 
  Date: 15/07/2016
  Authors: European Commission, Joint Research Centre
  Daniele Francioli, Emanuela Epure
  inspire-registry-dev@jrc.ec.europa.eu
 
This work was supported by the EU  Interoperability Solutions for
European Public Administrations Programme (http://ec.europa.eu/isa)
through Action 1.17: Re-usable INSPIRE Reference Platform 
(http://ec.europa.eu/isa/actions/01-trusted-information-exchange/1-17action_en.htm).-->

<xsl:stylesheet version="2.0" 
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
                xmlns="http://localhost/company_register/company"> 
        
    <!--Output settings-->
    <xsl:output method="xml" media-type="application/xml" version="1.0" encoding="UTF-8" indent="yes"/>

    <!--Variables & parameters declaration-->
    <xsl:param name="base_db_export_path" />
    <xsl:variable name="register_uri" select="item/register/uri"/>
    
    <xsl:template match="/">
        <company  xsi:schemaLocation="http://localhost/company_register/company http://localhost/PATH_SCHEMA/Company.xsd">
            <xsl:apply-templates select="item"/>
        </company>
    </xsl:template>
    
    <xsl:template match="item">
        
        <xsl:attribute name="id">
            <xsl:value-of select="uri"/>
        </xsl:attribute>
         
        <xsl:if test="thisversion != ''">
            <xsl:element name="thisversion" >
                <xsl:value-of select="thisversion/uri" />
            </xsl:element>
        </xsl:if>
        
        <xsl:if test="latestversion != ''">
            <xsl:element name="latestversion" >
                <xsl:value-of select="latestversion/uri" />
            </xsl:element>
        </xsl:if>
          
        <xsl:if test="previousversions != ''">
            <xsl:element name="previousversions" >
                <xsl:for-each select="previousversions/version">
                    <xsl:element name="version">
                        <xsl:value-of select="uri"/>         
                    </xsl:element>                
                </xsl:for-each>
            </xsl:element>
        </xsl:if>
        
        <xsl:variable name="language" select="language/isocode"></xsl:variable>  
        <xsl:element name="language" >
            <xsl:value-of select="$language" />
        </xsl:element>
        
        <xsl:if test="label != ''">
            <xsl:element name="label" >
                <xsl:attribute name="xml:lang">
                    <xsl:value-of select="label/@language"/>
                </xsl:attribute>
                <xsl:value-of select="label" />
            </xsl:element>
        </xsl:if>
        
        <xsl:if test="definition != ''">
            <xsl:variable name="tmp" select="definition"/>
            
            <xsl:element name="definition" >
                <xsl:attribute name="xml:lang">
                    <xsl:value-of select="definition/@language"/>
                </xsl:attribute>
                <xsl:value-of select="$tmp" />
            </xsl:element>
        </xsl:if>
        
        <xsl:if test="description != ''">
            <xsl:variable name="tmp" select="description"/>
            
            <xsl:element name="description" >
                <xsl:attribute name="xml:lang">
                    <xsl:value-of select="description/@language"/>
                </xsl:attribute>
                <xsl:value-of select="$tmp" />
            </xsl:element>        
        </xsl:if>
        
        <xsl:if test="customattributes/customattribute[name/text()='Country'] != ''">            
            <xsl:if test="customattributes/customattribute[name/text()='Country']/values != ''">            
                <xsl:for-each select="customattributes/customattribute[name/text()='Country']/values/value">                   
                    <xsl:variable name="CountryUri" select="uri"></xsl:variable>
                    <xsl:element name="country">
                        <xsl:attribute name="id">
                            <xsl:value-of select="$CountryUri"/>
                        </xsl:attribute>
                        
                        <xsl:if test="label">
                            <xsl:variable name="CountryLabel" select="label"></xsl:variable>
                            <xsl:variable name="CountryLabelLanguage" select="label/@language"></xsl:variable>
                            <label>
                                <xsl:attribute name="xml:lang">
                                    <xsl:value-of select="$CountryLabelLanguage"/>
                                </xsl:attribute>
                                <xsl:value-of select="$CountryLabel"/>
                            </label>
                        </xsl:if>
                    </xsl:element>              
                </xsl:for-each>
            </xsl:if>
        </xsl:if>
               
        <xsl:if test="customattributes/customattribute[name/text()='Website'] != ''">            
            <xsl:if test="customattributes/customattribute[name/text()='Website']/values != ''">            
                <xsl:for-each select="customattributes/customattribute[name/text()='Website']/values/value">                   
                    <xsl:variable name="WebsiteUriname" select="uriname"></xsl:variable>
                    <xsl:element name="website">
                        <label>
                            <xsl:attribute name="xml:lang">
                                <xsl:value-of select="$language"/>
                            </xsl:attribute>
                            <xsl:value-of select="$WebsiteUriname"/>
                        </label>
                    </xsl:element>              
                </xsl:for-each>
            </xsl:if>
        </xsl:if>
        
        <xsl:element name="itemclass" >
            <xsl:attribute name="uriname">
                <xsl:value-of select="itemclass/uriname"/>
            </xsl:attribute>
            
            <xsl:element name="label" >
                <xsl:attribute name="xml:lang">
                    <xsl:value-of select="itemclass/label/@language"/>
                </xsl:attribute>
                <xsl:value-of select="itemclass/label" />
            </xsl:element>
        </xsl:element>
        
        <xsl:element name="status" >
            <xsl:attribute name="id">
                <xsl:value-of select="status/baseuri"/>
                <xsl:text>/</xsl:text>
                <xsl:value-of select="status/uriname"/>
            </xsl:attribute>
            
            <xsl:element name="label" >
                <xsl:attribute name="xml:lang">
                    <xsl:value-of select="status/label/@language"/>
                </xsl:attribute>
                <xsl:value-of select="status/label" />
            </xsl:element>
        </xsl:element>
        
        <xsl:element name="register" >
            <xsl:attribute name="id">
                <xsl:value-of select="register/uri"/>
            </xsl:attribute>
            
            <label>
                <xsl:attribute name="xml:lang">
                    <xsl:value-of select="register/label/@language"/>
                </xsl:attribute>
                <xsl:value-of select="register/label"/>
            </label>
                                
            <xsl:element name="registry" >
                <xsl:attribute name="id">
                    <xsl:value-of select="register/registry/uri"/>
                </xsl:attribute>
            
                <label>
                    <xsl:attribute name="xml:lang">
                        <xsl:value-of select="register/registry/label/@language"/>
                    </xsl:attribute>
                    <xsl:value-of select="register/registry/label"/>
                </label>
            </xsl:element>
        </xsl:element>   
        
        <xsl:choose>
            <xsl:when test="predecessors">
                <xsl:element name="predecessors" >
                    <xsl:for-each select="predecessors/predecessor">
                
                        <xsl:element name="predecessor">
                            <xsl:attribute name="id">
                                <xsl:value-of select="uri"/>
                            </xsl:attribute>         
                            
                            <xsl:if test="label != ''">
                                <xsl:element name="label" >
                                    <xsl:attribute name="xml:lang">
                                        <xsl:value-of select="label/@language"/>
                                    </xsl:attribute>
                                    <xsl:value-of select="label" />
                                </xsl:element>
                            </xsl:if>
                                              
                        </xsl:element>                
                    </xsl:for-each>
                </xsl:element>
            </xsl:when>
            <xsl:otherwise>
            </xsl:otherwise>
        </xsl:choose>
        
        <xsl:choose>
            <xsl:when test="successors">
                <xsl:element name="successors" >
                    <xsl:for-each select="successors/successor">
                
                        <xsl:element name="successor">
                            <xsl:attribute name="id">
                                <xsl:value-of select="uri"/>
                            </xsl:attribute>         
                            
                            <xsl:if test="label != ''">
                                <xsl:element name="label" >
                                    <xsl:attribute name="xml:lang">
                                        <xsl:value-of select="label/@language"/>
                                    </xsl:attribute>
                                    <xsl:value-of select="label" />
                                </xsl:element>
                            </xsl:if>
                                              
                        </xsl:element>                
                    </xsl:for-each>
                </xsl:element>
            </xsl:when>
            <xsl:otherwise>
            </xsl:otherwise>
        </xsl:choose>
        
    </xsl:template>
</xsl:stylesheet>