/*
 * Copyright 2010,2015 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 15/07/2016
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-dev@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistry.module;

import eu.europa.ec.re3gistry.constants.Constants;
import eu.europa.ec.re3gistrycommon.utils.Configuration;
import java.util.Locale;
import java.util.Properties;
import java.util.ResourceBundle;

public class Module {

    private Properties properties;
    private ResourceBundle localization;

    private String moduleName;
    private String menuLabel;

    private boolean moduleActive;
    private boolean viewInMenu;
    private int menuOrder;
    private boolean homeWidgetEnabled;

    public Module(String moduleName, Locale locale) {
        this.moduleName = moduleName;

        //Getting properties
        Properties props = Configuration.getInstance().getProperties(moduleName);
        this.properties = props;

        //Setting value from properties
        this.moduleActive = (props.getProperty(Constants.MODULE_KEY_ACTIVE, "false").equals("true"));
        this.viewInMenu = (props.getProperty(Constants.MODULE_KEY_MENU_VISIBLE, "false").equals("true"));
        this.homeWidgetEnabled = (props.getProperty(Constants.MODULE_KEY_HOMEWIDGET_ENABLED, "false").equals("true"));
        this.menuLabel = props.getProperty(Constants.MODULE_KEY_MENUITEM_LABEL, moduleName);
        this.menuOrder = Integer.parseInt(props.getProperty(Constants.MODULE_KEY_MENUITEM_ORDER, "0"));

        //Getting localization
        this.localization = ResourceBundle.getBundle(Constants.LOCALIZATION_PREFIX + moduleName + Constants.LOCALIZATION_SUFFIX, locale);
    }

    public String getModuleName() {
        return moduleName;
    }

    public void setModuleName(String moduleName) {
        this.moduleName = moduleName;
    }

    public String getMenuLabel() {
        return menuLabel;
    }

    public void setMenuLabel(String menuLabel) {
        this.menuLabel = menuLabel;
    }

    public boolean isModuleActive() {
        return moduleActive;
    }

    public void setModuleActive(boolean moduleActive) {
        this.moduleActive = moduleActive;
    }

    public boolean isViewInMenu() {
        return viewInMenu;
    }

    public void setViewInMenu(boolean viewInMenu) {
        this.viewInMenu = viewInMenu;
    }

    public boolean isHomeWidgetEnabled() {
        return homeWidgetEnabled;
    }

    public void setHomeWidgetEnabled(boolean homeWidgetEnabled) {
        this.homeWidgetEnabled = homeWidgetEnabled;
    }

    public Properties getProperties() {
        return properties;
    }

    public void setProperties(Properties prop) {
        this.properties = prop;
    }

    public ResourceBundle getLocalization() {
        return localization;
    }

    public void setLocalization(ResourceBundle localization) {
        this.localization = localization;
    }

    public int getMenuOrder() {
        return menuOrder;
    }

    public void setMenuOrder(int menuOrder) {
        this.menuOrder = menuOrder;
    }

    @Override
    public boolean equals(Object object) {
        boolean sameSame = false;
        if (object != null && object instanceof Module) {
            sameSame = this.moduleName.equals(((Module) object).moduleName);
        }
        return sameSame;
    }

    @Override
    public int hashCode() {
        int hash = 7;
        return hash;
    }

}
