/*
 * Copyright 2010,2015 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 15/07/2016
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-dev@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrycommon.model;

import eu.europa.ec.re3gistrycommon.utils.StringUtils;
import java.io.Serializable;
import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Table;
import javax.xml.bind.annotation.XmlRootElement;

@Entity
@Table(name = "itemclasscustomattribute")
@XmlRootElement
@NamedQueries({
    @NamedQuery(name = "Itemclasscustomattribute.findAll", query = "SELECT i FROM Itemclasscustomattribute i"),
    @NamedQuery(name = "Itemclasscustomattribute.findByUuid", query = "SELECT i FROM Itemclasscustomattribute i WHERE i.uuid = :uuid"),
    /**
     * * custom query*
     */
    @NamedQuery(name = "Itemclasscustomattribute.findAllByItemclass", query = "SELECT i FROM Itemclasscustomattribute i WHERE i.itemclass = :itemclass"),
    @NamedQuery(name = "Itemclasscustomattribute.findByItemclassCustomattribute", query = "SELECT i FROM Itemclasscustomattribute i WHERE i.itemclass = :itemclass AND i.customattribute = :customattribute")
})
public class Itemclasscustomattribute implements Serializable {

    private static final long serialVersionUID = 1L;
    @Id
    @Basic(optional = false)
    @Column(name = "uuid")
    private String uuid;
    @JoinColumn(name = "itemclass", referencedColumnName = "uuid")
    @ManyToOne(optional = false)
    private Itemclass itemclass;
    @JoinColumn(name = "customattribute", referencedColumnName = "uuid")
    @ManyToOne(optional = false)
    private Customattribute customattribute;
    @Basic(optional = false)
    @Column(name = "required")
    private boolean required;
    @Basic(optional = false)
    @Column(name = "isforeignkey")
    private boolean isforeignkey;

    public Itemclasscustomattribute() {
    }

    public Itemclasscustomattribute(String uuid) {
        this.uuid = uuid;
    }

    public String getUuid() {
        return uuid;
    }

    public void setUuid(String uuid) {
        this.uuid = uuid;
    }

    public Itemclass getItemclass() {
        return itemclass;
    }

    public void setItemclass(Itemclass itemclass) {
        this.itemclass = itemclass;
    }

    public Customattribute getCustomattribute() {
        return customattribute;
    }

    public void setCustomattribute(Customattribute customattribute) {
        this.customattribute = customattribute;
    }

    @Override
    public int hashCode() {
        int hash = 0;
        hash += (uuid != null ? uuid.hashCode() : 0);
        return hash;
    }

    public boolean getRequired() {
        return required;
    }

    public void setRequired(boolean required) {
        this.required = required;
    }

    public boolean getIsforeignkey() {
        return isforeignkey;
    }

    public void setIsforeignkey(boolean isforeignkey) {
        this.isforeignkey = isforeignkey;
    }

    @Override
    public boolean equals(Object object) {
        if (!(object instanceof Itemclasscustomattribute)) {
            return false;
        }
        Itemclasscustomattribute other = (Itemclasscustomattribute) object;
        return (this.uuid != null || other.uuid == null) && (this.uuid == null || this.uuid.equals(other.uuid));
    }

    @Override
    public String toString() {
        return "eu.europa.ec.re3gistrycommon.model.Itemclasscustomattribute[ uuid=" + uuid + " ]";
    }

    /**
     * Creates the uuid for the Itemclasscustomattribute's objects
     *
     * @param customAttributeDescriptorName The customAttributeDescriptorName
     * @param itemItemclass the item's itemclass
     * @return the uuid
     * @throws Exception
     */
    public static String createUuid(String customAttributeDescriptorName, String itemItemclass) throws Exception {
        return StringUtils.createUUID(customAttributeDescriptorName + itemItemclass, Itemclasscustomattribute.class);
    }

}
