/*
 * Copyright 2010,2015 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 15/07/2016
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-dev@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrycommon.model;

import eu.europa.ec.re3gistrycommon.utils.StringUtils;
import java.io.Serializable;
import java.util.Date;
import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EntityResult;
import javax.persistence.FieldResult;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.SqlResultSetMapping;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.xml.bind.annotation.XmlRootElement;

@Entity
@Table(name = "itemparent")
@XmlRootElement
@NamedQueries({
    @NamedQuery(name = "Itemparent.findAll", query = "SELECT i FROM Itemparent i"),
    @NamedQuery(name = "Itemparent.findByUuid", query = "SELECT i FROM Itemparent i WHERE i.uuid = :uuid"),
    @NamedQuery(name = "Itemparent.findByDatecreation", query = "SELECT i FROM Itemparent i WHERE i.datecreation = :datecreation"),
    @NamedQuery(name = "Itemparent.findByDatelastupdate", query = "SELECT i FROM Itemparent i WHERE i.datelastupdate = :datelastupdate"),
    /**
     * Custom query
     */
    @NamedQuery(name = "Itemparent.findByItem", query = "SELECT i FROM Itemparent i WHERE i.item = :item"),
    @NamedQuery(name = "Itemparent.findByParent", query = "SELECT i FROM Itemparent i WHERE i.parent = :parent")
})
@SqlResultSetMapping(
	name="itemparentResult",
	entities=@EntityResult(
		entityClass=Itemparent.class,
		fields={
			@FieldResult(name="uuid", column="uuid"),
                        @FieldResult(name="item", column="item"),
                        @FieldResult(name="parent", column="parent"),
                        @FieldResult(name="datecreation", column="datecreation"),
                        @FieldResult(name="datelastupdate", column="datelastupdate")
                        
		}
	)
)
public class Itemparent implements Serializable {

    private static final long serialVersionUID = 1L;
    @Id
    @Basic(optional = false)
    @Column(name = "uuid")
    private String uuid;
    @Basic(optional = false)
    @Column(name = "datecreation")
    @Temporal(TemporalType.TIMESTAMP)
    private Date datecreation;
    @Column(name = "datelastupdate")
    @Temporal(TemporalType.TIMESTAMP)
    private Date datelastupdate;
    @JoinColumn(name = "parent", referencedColumnName = "uuid")
    @ManyToOne(optional = false)
    private Item parent;
    @JoinColumn(name = "item", referencedColumnName = "uuid")
    @ManyToOne(optional = false)
    private Item item;

    public Itemparent() {
    }

    public Itemparent(String uuid) {
        this.uuid = uuid;
    }

    public Itemparent(String uuid, Date datecreation) {
        this.uuid = uuid;
        this.datecreation = datecreation;
    }

    public String getUuid() {
        return uuid;
    }

    public void setUuid(String uuid) {
        this.uuid = uuid;
    }

    public Date getDatecreation() {
        return datecreation;
    }

    public void setDatecreation(Date datecreation) {
        this.datecreation = datecreation;
    }

    public Date getDatelastupdate() {
        return datelastupdate;
    }

    public void setDatelastupdate(Date datelastupdate) {
        this.datelastupdate = datelastupdate;
    }

    public Item getParent() {
        return parent;
    }

    public void setParent(Item parent) {
        this.parent = parent;
    }

    public Item getItem() {
        return item;
    }

    public void setItem(Item item) {
        this.item = item;
    }

    @Override
    public int hashCode() {
        int hash = 0;
        hash += (uuid != null ? uuid.hashCode() : 0);
        return hash;
    }

    @Override
    public boolean equals(Object object) {
        if (!(object instanceof Itemparent)) {
            return false;
        }
        Itemparent other = (Itemparent) object;
        return (this.uuid != null || other.uuid == null) && (this.uuid == null || this.uuid.equals(other.uuid));
    }

    @Override
    public String toString() {
        return "eu.europa.ec.re3gistrycommon.model.Itemparent[ uuid=" + uuid + " ]";
    }

    /**
     * Creates the uuid for the Itemparent's objects
     *
     * @param itemUuid The item's uuid
     * @param parentUuid The parent item's uuid
     * @return the uuid
     * @throws Exception
     */
    public static String createUuid(String itemUuid, String parentUuid) throws Exception {
        return StringUtils.createUUID(itemUuid + parentUuid, Itemcollection.class);
    }
}
