/*
 * Copyright 2010,2015 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 15/07/2016
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-dev@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrydata.managers;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import net.lingala.zip4j.core.ZipFile;

public class ZipMgr {

    /**
     * This method unzip the file passed by parameter. Returns true if the unzip
     * process successfully end.
     *
     * @param outputDir the directory were the unzipped file will be stored
     * @param importFile the zip file to be unzipped
     * @return true if the unzip process successfully end
     * @throws Exception
     */
    public static boolean unzipFile(String outputDir, File file) throws Exception {
        byte[] buffer = new byte[1024];

        //Creating the outputFolder if it does not exist
        File folder = new File(outputDir);
        if (!folder.exists() && !folder.mkdir()) {
            throw new Exception("Error occurred while creating the temp output folder for unzipped files.");

        }

        try {
            FileInputStream fileInputStream = new FileInputStream(file);

            ZipInputStream zis = new ZipInputStream(fileInputStream);
            ZipEntry ze = zis.getNextEntry();

            while (ze != null) {
                String fileName = ze.getName();
                File newFile = new File(outputDir + File.separator + fileName);

                if (ze.isDirectory()) {
                    if (!newFile.exists() && !newFile.mkdir()) {
                        throw new Exception("Error occurred while creating folder(s) during the files unzip.");
                    }
                } else {
                    FileOutputStream fos = new FileOutputStream(newFile);
                    int len;
                    while ((len = zis.read(buffer)) > 0) {
                        fos.write(buffer, 0, len);
                    }

                    fos.close();
                }
                ze = zis.getNextEntry();
            }

            zis.closeEntry();
            zis.close();

            return true;
        } catch (Exception e) {
            try {
                ZipFile zipFile = new ZipFile(file);
                zipFile.extractAll(outputDir);
                return true;
            } catch (Exception ee) {
                throw new Exception(ee.getMessage());
            }
        }
    }
}
