/*
 * Copyright 2010,2015 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 15/07/2016
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-dev@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrystaticizer.marshaller;

import eu.europa.ec.re3gistrycommon.managers.LanguagecodeMgr;
import eu.europa.ec.re3gistrycommon.managers.LocalizationMgr;
import eu.europa.ec.re3gistrycommon.managers.ReportMgr;
import eu.europa.ec.re3gistrycommon.model.Languagecode;
import eu.europa.ec.re3gistrycommon.model.Localization;
import eu.europa.ec.re3gistrycommon.model.Register;
import eu.europa.ec.re3gistrystaticizer.StaticizeFiles;
import eu.europa.ec.re3gistrystaticizer.constants.Constants;
import eu.europa.ec.re3gistrystaticizer.constants.StaticizeUtil;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.RegisterStandard;
import java.io.File;
import javax.persistence.EntityManager;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.transform.Transformer;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import net.sf.saxon.TransformerFactoryImpl;

public class RegisterStaticizer {

    private final String folderSeparator;
    private final String folderStandardStaticizer;
    private final String folderCustomStaticizer;
    private final EntityManager em;
    private static org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger(Constants.MODULE_NAME);
    private final StaticizeFiles staticizeFiles;
    private final Languagecode masterLanguage;
    private final ReportMgr reportMgr;

    public RegisterStaticizer(StaticizeFiles staticizeFiles, EntityManager em) throws Exception {
        this.staticizeFiles = staticizeFiles;
        this.folderSeparator = staticizeFiles.getFolderSeparator();
        this.folderStandardStaticizer = staticizeFiles.getCreateStandardFiles().getFolderStandardStaticizer();
        this.folderCustomStaticizer = staticizeFiles.getCreateStandardFiles().getFolderCustomStaticizer();
        this.em = em;
        this.reportMgr = staticizeFiles.getReportMgr();

        LanguagecodeMgr languagecodeMgr = new LanguagecodeMgr(em);
        masterLanguage = languagecodeMgr.getMasterLanguagecode();
    }

    public File addRegisterFiles(Register register, Languagecode languagecode) throws Exception {
        if (staticizeFiles.getCreateStandardFiles().getEXTENSIONS().length != 0 && !staticizeFiles.getCreateStandardFiles().getEXTENSIONS()[0].equals("")) {
            try {
                LocalizationMgr localizationMgr = new LocalizationMgr(em);

                Localization registerLocalization;
                try {
                    registerLocalization = localizationMgr.findByRegisterAndLanguagecode(register, languagecode);
                } catch (Exception ex) {
                    registerLocalization = localizationMgr.findByRegisterAndLanguagecode(register, masterLanguage);
                }
                RegisterStandard registerStandard = StaticizeUtil.addRegisterStandard(register, registerLocalization, em);
                return createXMLRegisterFile(registerStandard, register);
            } catch (Exception ex) {
                String subject = staticizeFiles.getLocalizationres().getString("export.mail.subject.error");
                String body = staticizeFiles.getLocalizationres().getString("export.mail.body.error.creation.custom.item.files").replace("{user}", staticizeFiles.getDataprocedure().getUsers().getEmail());
                staticizeFiles.getReportMgr().setMailSubject(subject);
                staticizeFiles.getReportMgr().setMailBody(body);
                staticizeFiles.getReportMgr().setStatus(eu.europa.ec.re3gistrycommon.constants.Constants.ID_IMPORTED_STATUS);

                this.reportMgr.addReport("Error in writing Standard && Custom Item Files for the Register", subject, body, Constants.REPORT_TYPE_ERROR, -1);

                log.info("=================================Error in writing Standard && Custom Item Files for the Register=============================================" + ex.getMessage());
                throw new Exception("Has been encountered a problem while writing the custom item files for the Register. ");
            }
        }
        return null;
    }

    private File createXMLRegisterFile(RegisterStandard registerStandard, Register register) throws Exception {
        String isocode = registerStandard.getLabel().getLanguage();

        String registerFolderStandardPath = folderStandardStaticizer + folderSeparator + register.getUriname();
        String registerFolderCustomPath = folderCustomStaticizer + folderSeparator + register.getUriname();
        File registerStandardFolder = new File(registerFolderStandardPath);
        File registercustomFolder = new File(registerFolderCustomPath);

        if (!registerStandardFolder.exists()) {
            registerStandardFolder.mkdirs();
        }
        if (!registercustomFolder.exists()) {
            registercustomFolder.mkdirs();
        }

        String registerUriname = register.getUriname();
        String registerStandardFilePath = registerStandardFolder + folderSeparator + registerUriname + "." + isocode + ".xml";
        File registerStandardFile = new File(registerStandardFilePath);

        try {
            /**
             * STANDARD XML
             */
            JAXBContext jaxbContextLang = JAXBContext.newInstance(RegisterStandard.class);
            Marshaller marshallerclassLanguage = jaxbContextLang.createMarshaller();
            marshallerclassLanguage.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
            marshallerclassLanguage.marshal(registerStandard, registerStandardFile);
            log.info("Create with success: " + registerStandardFile);

            /**
             * CUSTOM XML, JSON, HTML, ATOM, RDF
             */
            for (String ext : staticizeFiles.getCreateStandardFiles().getEXTENSIONS()) {
                try {
                    String registerXSL = staticizeFiles.getStaticizerXSLFolder() + "/" + ext + "/" + registerUriname + "_register." + ext + ".xsl";
                    String registerCustomFilePath = registercustomFolder + folderSeparator + registerUriname + "." + isocode + "." + ext;

                    File finalOutFile = new File(registerCustomFilePath);
                    finalOutFile.createNewFile();
                    File transformerFile = new File(registerXSL);

                    TransformerFactoryImpl factory = new net.sf.saxon.TransformerFactoryImpl();
                    Transformer transformer = factory.newTransformer(new StreamSource(transformerFile));
                    transformer.transform(new StreamSource(registerStandardFile), new StreamResult(finalOutFile));

                    log.info("Create with success: " + finalOutFile);

                } catch (Exception ex) {
                    String subject = staticizeFiles.getLocalizationres().getString("export.mail.subject.error");
                    String body = staticizeFiles.getLocalizationres().getString("export.mail.body.error.creation.custom.item.files").replace("{user}", staticizeFiles.getDataprocedure().getUsers().getEmail());
                    staticizeFiles.getReportMgr().setMailSubject(subject);
                    staticizeFiles.getReportMgr().setMailBody(body);
                    staticizeFiles.getReportMgr().setStatus(eu.europa.ec.re3gistrycommon.constants.Constants.ID_IMPORTED_STATUS);

                    this.reportMgr.addReport("Error in writing Standard && Custom Item Files", subject, body, Constants.REPORT_TYPE_ERROR, -1);

                    log.info("=================================Error in writing Standard && Custom Item Files=============================================" + ex.getMessage());
                    throw new Exception("Has been encountered a problem while writing the custom item files. ");
                }
            }
        } catch (JAXBException ex) {
            String subject = staticizeFiles.getLocalizationres().getString("export.mail.subject.error");
            String body = staticizeFiles.getLocalizationres().getString("export.mail.body.error.creation.custom.item.files").replace("{user}", staticizeFiles.getDataprocedure().getUsers().getEmail());
            staticizeFiles.getReportMgr().setMailSubject(subject);
            staticizeFiles.getReportMgr().setMailBody(body);
            staticizeFiles.getReportMgr().setStatus(eu.europa.ec.re3gistrycommon.constants.Constants.ID_IMPORTED_STATUS);

            this.reportMgr.addReport("Error in writing Standard && Custom Item Files", subject, body, Constants.REPORT_TYPE_ERROR, -1);

            log.info("=================================Error in writing Standard && Custom Item Files=============================================" + ex.getMessage());
            throw new Exception("Has been encountered a problem while writing the custom item files. ");
        }

        return registercustomFolder;
    }
}
