/*
 * Copyright 2010,2014 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 15/07/2016
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-dev@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrystaticizer.marshaller;

import eu.europa.ec.re3gistrycommon.managers.LocalizationMgr;
import eu.europa.ec.re3gistrycommon.model.Item;
import eu.europa.ec.re3gistrycommon.model.Itemclass;
import eu.europa.ec.re3gistrycommon.model.Languagecode;
import eu.europa.ec.re3gistrycommon.model.Localization;
import eu.europa.ec.re3gistrycommon.model.Register;
import eu.europa.ec.re3gistrystaticizer.StaticizeFiles;
import eu.europa.ec.re3gistrystaticizer.util.Constants;
import eu.europa.ec.re3gistrystaticizer.util.RegisterUtil;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.ItemStandard;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.RegisterStandard;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.RegistryStandard;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.collection.CollectionItemStandard;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.collection.CollectionRegisterStandard;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.collection.CollectionRegistryStandard;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.collection.ContainerCollectionItemStandard;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.collection.ContainerCollectionRegisterStandard;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.collection.ContainerCollectionRegistryStandard;
import eu.europa.ec.re3gistrystaticizer.util.ItemUtil;
import eu.europa.ec.re3gistrystaticizer.marshaller.solr.ItemsSolrFiles;
import eu.europa.ec.re3gistrystaticizer.marshaller.solr.ItemsSolrFiles;
import eu.europa.ec.re3gistrystaticizer.marshaller.solr.RegisterSolrFiles;
import eu.europa.ec.re3gistrystaticizer.marshaller.solr.RegisterSolrFiles;
import eu.europa.ec.re3gistrystaticizer.marshaller.solr.RegistrySolrFiles;
import eu.europa.ec.re3gistrystaticizer.marshaller.solr.RegistrySolrFiles;
import java.io.File;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import javax.persistence.EntityManager;

/**
 *
 * @author epuread
 */
public class SOLRFilesCreation {

    private String[] EXTENSIONSSOLR;

    private String folderSolrStaticizer;
    private String folderSolrCustomStaticizer;
    private String staticizerSolrStandardFolder;
    private String staticizerSolrCustomFolder;

    private final StaticizeFiles staticizeFiles;
    private final String folderSeparator = System.getProperty("file.separator");
    private final EntityManager em;

    public SOLRFilesCreation(StaticizeFiles staticizeFiles, LinkedHashMap<Item, List<Languagecode>> itemsTostaticize, LinkedHashMap<Itemclass, List<Languagecode>> registerToStaticize, EntityManager em) throws Exception {
        this.staticizeFiles = staticizeFiles;
        this.em = em;

        initalizateSOLRFolderAndSubFolders();

        if (EXTENSIONSSOLR.length != 0 && !EXTENSIONSSOLR[0].equals("")) {
            createSOLRFiles(itemsTostaticize, registerToStaticize);
        }
    }

    private void initalizateSOLRFolderAndSubFolders() {
        /**
         * initialize the SOLR standard and custom folder
         */
        Properties properties = eu.europa.ec.re3gistrycommon.utils.Configuration.getInstance().getProperties(Constants.MODULE_NAME);

        staticizerSolrStandardFolder = properties.getProperty(Constants.STATICIZER_SOLR_FOLDER, "solr");
        folderSolrStaticizer = staticizeFiles.getStaticizerMainfolderName() + folderSeparator + staticizerSolrStandardFolder;
        new File(folderSolrStaticizer).mkdirs();

        staticizerSolrCustomFolder = properties.getProperty(Constants.STATICIZER_SOLR_CUSTOM_FOLDER, "solrcustom");
        folderSolrCustomStaticizer = staticizeFiles.getStaticizerMainfolderName() + folderSeparator + staticizerSolrCustomFolder;
        new File(folderSolrCustomStaticizer).mkdirs();

        EXTENSIONSSOLR = new String[]{properties.getProperty(Constants.STATICIZER_SOLR_FORMAT)};
    }

    private void createSOLRFiles(LinkedHashMap<Item, List<Languagecode>> itemsTostaticize, LinkedHashMap<Itemclass, List<Languagecode>> registerToStaticize) throws Exception {
        /**
         * Generate the SOLR standard/custom files
         */
        LocalizationMgr localizationMgr = new LocalizationMgr(em);
        ItemUtil createItemStandardUtil = new ItemUtil(em);
        ItemsSolrFiles solrLastImportedItems = new ItemsSolrFiles(this, em);

        for (Map.Entry<Item, List<Languagecode>> entry : itemsTostaticize.entrySet()) {
            Item item = entry.getKey();

            CollectionItemStandard collectionItemStandard = new CollectionItemStandard();
            ArrayList<ItemStandard> arrayListItemStandard = new ArrayList<ItemStandard>();

            List<Localization> localizationList = localizationMgr.findByItem(item);
            for (Localization localization : localizationList) {
                ItemStandard itemStandard = createItemStandardUtil.createItemStandardSOLR(item, localization, localization.getLanguage());
                if (itemStandard.getVersionnumber() == 0) {
                    arrayListItemStandard.add(itemStandard);
                }
            }
            if (arrayListItemStandard.size() > 0) {
                collectionItemStandard.setItemStandard(arrayListItemStandard);

                ContainerCollectionItemStandard containerCollectionItemStandard = new ContainerCollectionItemStandard();
                containerCollectionItemStandard.setCollectionItemStandard(collectionItemStandard);

                solrLastImportedItems.createItemStandardSOLRFile(item, containerCollectionItemStandard);
            }
        }
        /**
         * SOLR registry
         */
        RegistrySolrFiles solrRegistryStaticizer = new RegistrySolrFiles(this, em);
        CollectionRegistryStandard collectionRegistryStandard = new CollectionRegistryStandard();
        ArrayList<RegistryStandard> registryStandardaArrayList = solrRegistryStaticizer.addRegistryFiles();
        collectionRegistryStandard.setRegistryStandard(registryStandardaArrayList);
        ContainerCollectionRegistryStandard containerCollectionRegistryStandard = new ContainerCollectionRegistryStandard();
        containerCollectionRegistryStandard.setCollectionRegistryStandard(collectionRegistryStandard);
        solrRegistryStaticizer.createSOLRRegistryFile(containerCollectionRegistryStandard);

        /**
         * SOLR register
         */
        RegisterSolrFiles solrRegisterStaticizer = new RegisterSolrFiles(this);
        for (Map.Entry<Itemclass, List<Languagecode>> entry : registerToStaticize.entrySet()) {
            Itemclass itemclass = entry.getKey();
            Register register = itemclass.getRegister();

            CollectionRegisterStandard collectionRegisterStandard = new CollectionRegisterStandard();
            ArrayList<RegisterStandard> registerStaticizerArrayList = new ArrayList<RegisterStandard>();

            List<Localization> localizationList = localizationMgr.findRegisterLocalization(register);
            for (Localization localization : localizationList) {
//                RegisterStaticizer registerStaticizer = new RegisterStaticizer(staticizeSOLRFiles, em);
                registerStaticizerArrayList.add(RegisterUtil.addRegisterStandard(register, localization, em, localization.getLanguage()));
            }
            collectionRegisterStandard.setRegisterStandard(registerStaticizerArrayList);

            ContainerCollectionRegisterStandard containerCollectionRegisterStandard = new ContainerCollectionRegisterStandard();
            containerCollectionRegisterStandard.setCollectionRegisterStandard(collectionRegisterStandard);

            solrRegisterStaticizer.createSOLRRegisterFile(containerCollectionRegisterStandard, register);
        }
    }

    public String[] getEXTENSIONSSOLR() {
        return EXTENSIONSSOLR;
    }

    public String getFolderSolrStaticizer() {
        return folderSolrStaticizer;
    }

    public String getFolderSolrCustomStaticizer() {
        return folderSolrCustomStaticizer;
    }

    public StaticizeFiles getStaticizeFiles() {
        return staticizeFiles;
    }

}
