/*
 * Copyright 2010,2015 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 15/07/2016
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-dev@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrystaticizer.marshaller;

import eu.europa.ec.re3gistrycommon.managers.LanguagecodeMgr;
import eu.europa.ec.re3gistrycommon.managers.LocalizationMgr;
import eu.europa.ec.re3gistrycommon.managers.RegisterMgr;
import eu.europa.ec.re3gistrycommon.managers.RegistryMgr;
import eu.europa.ec.re3gistrycommon.model.Languagecode;
import eu.europa.ec.re3gistrycommon.model.Localization;
import eu.europa.ec.re3gistrycommon.model.Reference;
import eu.europa.ec.re3gistrycommon.model.Register;
import eu.europa.ec.re3gistrycommon.model.Registry;
import eu.europa.ec.re3gistrycommon.utils.StringUtils;
import eu.europa.ec.re3gistrystaticizer.constants.Constants;
import eu.europa.ec.re3gistrystaticizer.constants.StaticizeUtil;
import eu.europa.ec.re3gistrystaticizer.filescreation.CreateSOLRFiles;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.RegisterStandard;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.RegistryStandard;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.attribute.DefinitionAttribute;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.attribute.DescriptionAttribute;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.attribute.LabelAttribute;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.attribute.ReferenceAttribute;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.collection.ContainerCollectionRegistryStandard;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.list.RegisterStandardList;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.persistence.EntityManager;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import net.sf.saxon.TransformerFactoryImpl;

public class SolrRegistryStaticizer {

    private final EntityManager em;

    private final String folderSolrStaticizer;
    private final String folderSolrcustomStaticizer;
    private final CreateSOLRFiles createSOLRFiles;

    private final String folderSeparator = System.getProperty("file.separator");

    private static org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger(Constants.MODULE_NAME);
    private final Languagecode masterLanguage;

    /**
     *
     * @param createSOLRFiles
     * @param em
     */
    public SolrRegistryStaticizer(CreateSOLRFiles createSOLRFiles, EntityManager em) throws Exception {
        this.createSOLRFiles = createSOLRFiles;
        this.folderSolrStaticizer = createSOLRFiles.getFolderSolrStaticizer();
        this.folderSolrcustomStaticizer = createSOLRFiles.getFolderSolrCustomStaticizer();

        this.em = em;

        LanguagecodeMgr languagecodeMgr = new LanguagecodeMgr(em);
        masterLanguage = languagecodeMgr.getMasterLanguagecode();
    }

    public ArrayList<RegistryStandard> addRegistryFiles() throws Exception {
//        RegistryStaticizer registryStaticizer = new RegistryStaticizer(createSOLRFiles.getStaticizeFiles(), em);
        ArrayList<RegistryStandard> registryStandardArrayList = new ArrayList<RegistryStandard>();
        try {
            RegistryMgr registryMgr = new RegistryMgr(em);
            List<Registry> registryList = registryMgr.findAll();

            for (Registry registry : registryList) {
                LocalizationMgr localizationMgr = new LocalizationMgr(em);
                List<Localization> registryLocalizationList = localizationMgr.findRegistryLocalization(registry);

                for (Localization registryLocalization : registryLocalizationList) {
                    registryStandardArrayList.add(createRegistry(registry, registryLocalization));
                }
            }
        } catch (Exception ex) {
            log.error(ex.getMessage());
        }
        return registryStandardArrayList;
    }

    public File createSOLRRegistryFile(ContainerCollectionRegistryStandard containerCollectionRegistryStandard) {
        String registrySolrFolderPath = folderSolrStaticizer;
        String registrySolrcustomFolderPath = folderSolrcustomStaticizer;
        File registrySolrFolder = new File(registrySolrFolderPath);
        File registrySolrcustomFolder = new File(registrySolrcustomFolderPath);

        if (!registrySolrFolder.exists()) {
            registrySolrFolder.mkdirs();
        }
        if (!registrySolrcustomFolder.exists()) {
            registrySolrcustomFolder.mkdirs();
        }

        String registry = "registry";
        String registryFilePath = registrySolrFolder.getAbsolutePath() + folderSeparator + registry + ".xml";
        File registryFile = new File(registryFilePath);

        try {
            /**
             * STANDARD XML
             */
            JAXBContext jaxbContextLang = JAXBContext.newInstance(ContainerCollectionRegistryStandard.class);
            Marshaller marshallerclassLanguage = jaxbContextLang.createMarshaller();
            marshallerclassLanguage.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
            marshallerclassLanguage.marshal(containerCollectionRegistryStandard, registryFile);
            log.info("Create with success: " + registryFile);

            /**
             * CUSTOM SOLR
             */
            for (String ext : createSOLRFiles.getEXTENSIONSSOLR()) {
                try {
                    String registryXSL = createSOLRFiles.getStaticizeFiles().getStaticizerXSLFolder() + "/" + ext + "/" + registry + "." + ext + ".xsl";
//                    String finalOutputPath = registryFile.getAbsolutePath().replace("solr", "solrcustom");
                    String finalOutputPath = registrySolrcustomFolder + folderSeparator + registry + ".xml";

                    File finalOutFile = new File(finalOutputPath);
                    finalOutFile.createNewFile();
                    File transformerFile = new File(registryXSL);

                    TransformerFactoryImpl factory = new net.sf.saxon.TransformerFactoryImpl();
                    Transformer transformer = factory.newTransformer(new StreamSource(transformerFile));
                    transformer.transform(new StreamSource(registryFile), new StreamResult(finalOutFile));

                    log.info("Create with success: " + finalOutFile);
                } catch (IOException ex) {
                    log.error("Error in creation the transformation file: " + ex.getMessage());
                } catch (TransformerException ex) {
                    log.error("Error in creation the transformation file: " + ex.getMessage());
                }
            }

        } catch (JAXBException ex) {
            log.error("Error in creation file: " + registryFile + " -JAXBException- " + ex.getMessage());
        }

        return registrySolrcustomFolder;
    }

    /**
     * addition files
     *
     * @param registry
     * @param registryLocalization
     * @return
     * @throws java.lang.Exception
     */
    public RegistryStandard createRegistry(Registry registry, Localization registryLocalization) throws Exception {
        RegistryStandard registryStandard = new RegistryStandard();
        registryStandard.setUuid(registry.getUuid());

        registryStandard.setBaseuri(registry.getBaseuri());
        registryStandard.setUriname(registry.getUriname());
        registryStandard.setUri(registry.getBaseuri() + "/" + registry.getUriname());
        registryStandard.setUristandard(registry.getBaseuri() + "/" + registry.getUriname());

        String language = registryLocalization.getLanguage().getIsocode();

        registryStandard.setLanguage(registryLocalization.getLanguage());

        LabelAttribute registryLabelAttribute = new LabelAttribute();
        registryLabelAttribute.setLanguage(language);
        registryLabelAttribute.setValue(registryLocalization.getLabel());
        registryStandard.setLabel(registryLabelAttribute);

        StringUtils stringUtils = new StringUtils();

        if (registryLocalization.getDefinition() != null) {
            DefinitionAttribute registryDefinitionAttribute = new DefinitionAttribute();
            registryDefinitionAttribute.setLanguage(language);

            String definition = stringUtils.replaceSpecialCharacters(registryLocalization.getDefinition());
            registryDefinitionAttribute.setValue(definition);
            registryStandard.setDefinition(registryDefinitionAttribute);
        }

        if (registryLocalization.getDescription() != null) {
            DescriptionAttribute registryDescriptionAttribute = new DescriptionAttribute();
            registryDescriptionAttribute.setLanguage(language);

            String description = stringUtils.replaceSpecialCharacters(registryLocalization.getDescription());
            registryDescriptionAttribute.setValue(description);
            registryStandard.setDescription(registryDescriptionAttribute);
        }

        LocalizationMgr localizationMgr = new LocalizationMgr(em);
        Reference registryManager = registry.getRegistrymanager();
        ReferenceAttribute registermanagerReferenceAttribute = new ReferenceAttribute();
        registermanagerReferenceAttribute.setUuid(registryManager.getUuid());
        registermanagerReferenceAttribute.setUri(registryLocalization.getUri());
        registermanagerReferenceAttribute.setEmail(registryManager.getEmail());
        registermanagerReferenceAttribute.setReftype(registryManager.getReftype());
        registermanagerReferenceAttribute.setDatecreation(registryManager.getDatecreation());
        registermanagerReferenceAttribute.setDatelastupdate(registryManager.getDatelastupdate());

        String registerLabelFromLocaliation;
        LabelAttribute labelAttribute = new LabelAttribute();
        try {
            registerLabelFromLocaliation = (localizationMgr.findByReferenceAndLanguagecode(registryManager, registryLocalization.getLanguage())).getLabel();
            labelAttribute.setLanguage(registryLocalization.getLanguage().getIsocode());
        } catch (Exception ex) {
            registerLabelFromLocaliation = (localizationMgr.findByReferenceAndLanguagecode(registryManager, masterLanguage)).getLabel();
            labelAttribute.setLanguage(masterLanguage.getIsocode());
        }
        labelAttribute.setValue(registerLabelFromLocaliation);
        registermanagerReferenceAttribute.setLabel(labelAttribute);
        registryStandard.setRegistrymanager(registermanagerReferenceAttribute);

        registryStandard.setDatecreation(registry.getDatecreation());
        registryStandard.setDatelastupdate(registry.getDatelastupdate());

        RegisterStandardList registerStandardList = new RegisterStandardList();
        ArrayList<RegisterStandard> registerStandardArrayList = new ArrayList<RegisterStandard>();
        RegisterMgr registerMgr = new RegisterMgr(em);
        try {
            List<Register> registerList = registerMgr.findAll();

            for (Register register : registerList) {
                Localization registerLocalization;
                try {
                    registerLocalization = localizationMgr.findByRegisterAndLanguagecode(register, registryLocalization.getLanguage());
                } catch (Exception ex) {
                    registerLocalization = localizationMgr.findByRegisterAndLanguagecode(register, masterLanguage);
                    language = masterLanguage.getIsocode();
                }

                RegisterStandard registerStandard = StaticizeUtil.addRegisterStandard(register, registryLocalization.getLanguage(), registerLocalization, registryLabelAttribute, registry, em);
                registerStandardArrayList.add(registerStandard);
            }
        } catch (Exception ex) {
            Logger.getLogger(RegistryStaticizer.class.getName()).log(Level.SEVERE, null, ex);
        }
        registerStandardList.setRegisterStandard(registerStandardArrayList);
        registryStandard.setRegisterStandardList(registerStandardList);

        return registryStandard;
    }

}
