/*
 * Copyright 2010,2015 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 15/07/2016
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-dev@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrystaticizer.marshaller.custom;

import eu.europa.ec.re3gistrycommon.managers.ItemMgr;
import eu.europa.ec.re3gistrycommon.managers.ItemcollectionMgr;
import eu.europa.ec.re3gistrycommon.managers.ReportMgr;
import eu.europa.ec.re3gistrycommon.model.Item;
import eu.europa.ec.re3gistrycommon.model.Itemcollection;
import eu.europa.ec.re3gistrycommon.model.Languagecode;
import eu.europa.ec.re3gistrycommon.model.Localization;
import eu.europa.ec.re3gistrycommon.model.Register;
import eu.europa.ec.re3gistrystaticizer.StaticizeFiles;
import eu.europa.ec.re3gistrystaticizer.util.Constants;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.ItemStandard;
import eu.europa.ec.re3gistrystaticizer.util.ItemUtil;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import javax.persistence.EntityManager;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import net.sf.saxon.TransformerFactoryImpl;

public class ItemsCustomFiles {

    private final EntityManager em;
    private final StaticizeFiles staticizeFiles;
    private final String folderSeparator = System.getProperty("file.separator");
    private static final org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger(Constants.MODULE_NAME);
    private final ReportMgr reportMgr;

    public ItemsCustomFiles(StaticizeFiles staticizeFiles, EntityManager em) throws Exception {
        this.staticizeFiles = staticizeFiles;
        this.em = em;
        this.reportMgr = staticizeFiles.getReportMgr();
    }

    public ArrayList<File> createItemFiles(Item item, Localization localization, Languagecode languagecodeMissing) throws Exception {
        /**
         * create itemStandard
         */
        try {
            ItemUtil createItemStandardUtil = new ItemUtil(em);
            ItemStandard itemStandard = createItemStandardUtil.createItemStandard(item, localization, languagecodeMissing);
            return createItemStandardFile(item, languagecodeMissing.getIsocode(), itemStandard);
        } catch (Exception ex) {
            String subject = staticizeFiles.getLocalizationres().getString("export.mail.subject.error");
            String body = staticizeFiles.getLocalizationres().getString("export.mail.body.error.creation.custom.item.files").replace("{user}", staticizeFiles.getDataprocedure().getUsers().getEmail());
            staticizeFiles.getReportMgr().setMailSubject(subject);
            staticizeFiles.getReportMgr().setMailBody(body);
            staticizeFiles.getReportMgr().setStatus(eu.europa.ec.re3gistrycommon.constants.Constants.ID_IMPORTED_STATUS);

            this.reportMgr.addReport("Error in writing Standard && Custom Item Files for Items", subject, body, Constants.REPORT_TYPE_ERROR, -1);

            log.info("=================================Error in writing Standard && Custom Item Files for Items=============================================" + ex.getMessage());
            throw new Exception("Has been encountered a problem while writing the custom item files for Items. ");
        }
    }

    private ArrayList<File> createItemStandardFile(Item item, String isocode, ItemStandard itemStandard) throws Exception {
        ArrayList<File> files = new ArrayList<File>();
        Register register = item.getItemclass().getRegister();
        String registerUriname = register.getUriname();

        String folderStandardStaticizer = staticizeFiles.getCreateStandardFiles().getFolderStandardStaticizer();
        String folderCustomStaticiser = staticizeFiles.getCreateStandardFiles().getFolderCustomStaticizer();

        String registerStandardFolderPath = folderStandardStaticizer + folderSeparator + registerUriname;
        String registerCustomFolderPath = folderCustomStaticiser + folderSeparator + registerUriname;
        File registerStandardFolder = new File(registerStandardFolderPath);
        File registerCustomFolder = new File(registerCustomFolderPath);

        if (!registerStandardFolder.exists()) {
            registerStandardFolder.mkdirs();
        }
        if (!registerCustomFolder.exists()) {
            registerCustomFolder.mkdirs();
        }

        ItemcollectionMgr itemcollectionMgr = new ItemcollectionMgr(em);
        ItemMgr itemMgr = new ItemMgr(em);
        List<Item> previousVersions = itemMgr.getAllItemVersionOrderDescByVersionnumber(item);

        File fileStandard;
        File fileCustom = null;
        String itemVersionStandardFolderPath = null;
        String itemVersionCustomFolderPath = null;

        File latestFileStandard;
        File latestFileCustom = null;
        String latestItemVersionStandardFolderPath = null;
        String latestItemVersionCustomFolderPath = null;

        try {
            Itemcollection itemcollection = itemcollectionMgr.findByItem(item).get(0);

            if (itemcollection != null) {
                Item collection = itemcollection.getCollection();
                if (item.getVersionnumber() == 0) {
                    fileStandard = new File(registerStandardFolderPath + folderSeparator + collection.getUriname() + folderSeparator + item.getUriname());
                    fileCustom = new File(registerCustomFolderPath + folderSeparator + collection.getUriname() + folderSeparator + item.getUriname());
                    fileStandard.mkdirs();
                    fileCustom.mkdirs();
                    itemVersionStandardFolderPath = fileStandard.getAbsolutePath() + folderSeparator + item.getUriname() + "." + isocode + ".xml";
                    itemVersionCustomFolderPath = fileCustom.getAbsolutePath() + folderSeparator + item.getUriname() + "." + isocode + ".xml";

                    latestFileStandard = new File(registerStandardFolderPath + folderSeparator + collection.getUriname() + folderSeparator + item.getUriname() + folderSeparator + previousVersions.size());
                    latestFileCustom = new File(registerCustomFolderPath + folderSeparator + collection.getUriname() + folderSeparator + item.getUriname() + folderSeparator + previousVersions.size());
                    latestFileStandard.mkdirs();
                    latestFileCustom.mkdirs();
                    latestItemVersionStandardFolderPath = latestFileStandard.getAbsolutePath() + folderSeparator + item.getUriname() + "." + isocode + ".xml";
                    latestItemVersionCustomFolderPath = latestFileCustom.getAbsolutePath() + folderSeparator + item.getUriname() + "." + isocode + ".xml";
                } else {
                    fileStandard = new File(registerStandardFolderPath + folderSeparator + collection.getUriname() + folderSeparator + item.getUriname() + folderSeparator + item.getVersionnumber());
                    fileCustom = new File(registerCustomFolderPath + folderSeparator + collection.getUriname() + folderSeparator + item.getUriname() + folderSeparator + item.getVersionnumber());
                    fileStandard.mkdirs();
                    fileCustom.mkdirs();
                    itemVersionStandardFolderPath = fileStandard.getAbsolutePath() + folderSeparator + item.getUriname() + "." + isocode + ".xml";
                    itemVersionCustomFolderPath = fileCustom.getAbsolutePath() + folderSeparator + item.getUriname() + "." + isocode + ".xml";
                }
            }
        } catch (Exception ex) {
            if (item.getVersionnumber() == 0) {
                fileStandard = new File(registerStandardFolderPath + folderSeparator + item.getUriname());
                fileCustom = new File(registerCustomFolderPath + folderSeparator + item.getUriname());
                fileStandard.mkdirs();
                fileCustom.mkdirs();
                itemVersionStandardFolderPath = registerStandardFolderPath + folderSeparator + item.getUriname() + folderSeparator + item.getUriname() + "." + isocode + ".xml";
                itemVersionCustomFolderPath = registerCustomFolderPath + folderSeparator + item.getUriname() + folderSeparator + item.getUriname() + "." + isocode + ".xml";

                latestFileStandard = new File(registerStandardFolderPath + folderSeparator + item.getUriname() + folderSeparator + previousVersions.size());
                latestFileCustom = new File(registerCustomFolderPath + folderSeparator + folderSeparator + item.getUriname() + folderSeparator + previousVersions.size());
                latestFileStandard.mkdirs();
                latestFileCustom.mkdirs();
                latestItemVersionStandardFolderPath = latestFileStandard.getAbsolutePath() + folderSeparator + item.getUriname() + "." + isocode + ".xml";
                latestItemVersionCustomFolderPath = latestFileCustom.getAbsolutePath() + folderSeparator + item.getUriname() + "." + isocode + ".xml";
            } else {
                fileStandard = new File(registerStandardFolderPath + folderSeparator + item.getUriname() + folderSeparator + item.getVersionnumber());
                fileCustom = new File(registerCustomFolderPath + folderSeparator + item.getUriname() + folderSeparator + item.getVersionnumber());
                fileStandard.mkdirs();
                fileCustom.mkdirs();
                itemVersionStandardFolderPath = fileStandard.getAbsolutePath() + folderSeparator + item.getUriname() + "." + isocode + ".xml";
                itemVersionCustomFolderPath = fileCustom.getAbsolutePath() + folderSeparator + item.getUriname() + "." + isocode + ".xml";
            }
        }

        /**
         * write version 0
         */
        File itemVersionStandardFile = new File(itemVersionStandardFolderPath);
        File itemVersionCustomFile = new File(itemVersionCustomFolderPath);

        /**
         * create file in the version folder
         */
        try {
            createXMLFile(itemStandard, itemVersionStandardFile, itemVersionCustomFile);
        } catch (Exception ex) {
            log.info("=================================Error in writing Standard Item Files version folder for " + itemStandard.getItemclass() + "=============================================" + ex.getMessage());
            throw new Exception("Has been encountered a problem while writing the custom item files for version folder for " + itemStandard.getItemclass());
        }
        files.add(fileCustom);

        /**
         * write latest version
         */
        if (item.getVersionnumber() == 0) {
            File actualItemVersionStandardFile = new File(latestItemVersionStandardFolderPath);
            File actualItemVersionCustomFile = new File(latestItemVersionCustomFolderPath);

            try {
                createXMLFile(itemStandard, actualItemVersionStandardFile, actualItemVersionCustomFile);
            } catch (Exception ex) {
                log.info("=================================Error in writing Standard Item Files latest version folder for " + itemStandard.getItemclass() + "=============================================" + ex.getMessage());
                throw new Exception("Has been encountered a problem while writing the custom item files for latest version folder for " + itemStandard.getItemclass());
            }
            files.add(latestFileCustom);
        }

        return files;
    }

    private void createXMLFile(ItemStandard itemStandard, File itemStandardFile, File itemCustomFile) throws Exception {
        String itemFileUriname = itemStandard.getItemclass().getUriname();

        try {
            /**
             * STANDARD XML
             */
            JAXBContext jaxbContextLang = JAXBContext.newInstance(ItemStandard.class);
            Marshaller marshallerclassLanguage = jaxbContextLang.createMarshaller();
            marshallerclassLanguage.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
            marshallerclassLanguage.marshal(itemStandard, itemStandardFile);
            log.info("Create with success: " + itemStandardFile);

            /**
             * CUSTOM XML, JSON, HTML, ATOM, RDF, CSV, XMLISO19135
             */
            for (String ext : staticizeFiles.getCreateStandardFiles().getEXTENSIONS()) {
                transformXML(ext, itemFileUriname, itemCustomFile, itemStandardFile);
            }
        } catch (JAXBException ex) {
            String subject = staticizeFiles.getLocalizationres().getString("export.mail.subject.error");
            String body = staticizeFiles.getLocalizationres().getString("export.mail.body.error.creation.custom.item.files").replace("{user}", staticizeFiles.getDataprocedure().getUsers().getEmail());
            staticizeFiles.getReportMgr().setMailSubject(subject);
            staticizeFiles.getReportMgr().setMailBody(body);
            staticizeFiles.getReportMgr().setStatus(eu.europa.ec.re3gistrycommon.constants.Constants.ID_IMPORTED_STATUS);

            this.reportMgr.addReport("Error in writing Standard && Custom Item Files createXMLFile", subject, body, Constants.REPORT_TYPE_ERROR, -1);

            log.info("=================================Error in writing Standard && Custom Item Files createXMLFile=============================================" + ex.getMessage());
            throw new Exception("Has been encountered a problem while writing the custom item files createXMLFile. ");
        }
    }

    private void transformXML(String ext, String itemFileUriname, File itemCustomFile, File itemStandardFile) throws Exception {
        TransformerFactoryImpl factory = new net.sf.saxon.TransformerFactoryImpl();
        String itemFileXSL = staticizeFiles.getStaticizerXSLFolder() + "/" + ext + "/" + itemFileUriname + "." + ext + ".xsl";
        String finalOutputPath = itemCustomFile.getAbsolutePath().replace("xml", ext);

        try {
            if (new File(itemFileXSL).exists()) {
                File transformerFile = new File(itemFileXSL);
                File finalOutFile = new File(finalOutputPath);
                finalOutFile.createNewFile();
                Transformer transformer = factory.newTransformer(new StreamSource(transformerFile));
                transformer.transform(new StreamSource(itemStandardFile), new StreamResult(finalOutFile));

                log.info("Create with success: " + finalOutFile);
            }
        } catch (IOException ex) {
            String subject = staticizeFiles.getLocalizationres().getString("export.mail.subject.error");
            String body = staticizeFiles.getLocalizationres().getString("export.mail.body.error.creation.custom.item.files").replace("{user}", staticizeFiles.getDataprocedure().getUsers().getEmail());
            staticizeFiles.getReportMgr().setMailSubject(subject);
            staticizeFiles.getReportMgr().setMailBody(body);
            staticizeFiles.getReportMgr().setStatus(eu.europa.ec.re3gistrycommon.constants.Constants.ID_IMPORTED_STATUS);

            this.reportMgr.addReport("Error in writing Standard && Custom Item Files " + itemFileXSL + " does not exit", subject, body, Constants.REPORT_TYPE_ERROR, -1);

            log.info("=================================Error in writing Standard && Custom Item Files=============================================" + ex.getMessage() + " --- " + itemFileXSL + " does not exit");
            throw new Exception("Has been encountered a problem while writing the custom item files. " + itemFileXSL + " does not exit");
        } catch (TransformerException ex) {
            String subject = staticizeFiles.getLocalizationres().getString("export.mail.subject.error");
            String body = staticizeFiles.getLocalizationres().getString("export.mail.body.error.creation.custom.item.files").replace("{user}", staticizeFiles.getDataprocedure().getUsers().getEmail());
            staticizeFiles.getReportMgr().setMailSubject(subject);
            staticizeFiles.getReportMgr().setMailBody(body);
            staticizeFiles.getReportMgr().setStatus(eu.europa.ec.re3gistrycommon.constants.Constants.ID_IMPORTED_STATUS);

            this.reportMgr.addReport("Error in writing Standard && Custom Item Files " + itemFileXSL + " does not exit", subject, body, Constants.REPORT_TYPE_ERROR, -1);

            log.info("=================================Error in writing Standard && Custom Item Files=============================================" + ex.getMessage() + " --- " + itemFileXSL + " does not exit");
            throw new Exception("Has been encountered a problem while writing the custom item files. " + itemFileXSL + " does not exit");
        }
    }
}
