/*
 * Copyright 2010,2015 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 15/07/2016
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-dev@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrystaticizer.servlet;

import eu.europa.ec.re3gistrycommon.managers.DataprocedureMgr;
import eu.europa.ec.re3gistrycommon.managers.ReportMgr;
import eu.europa.ec.re3gistrycommon.model.Dataprocedure;
import eu.europa.ec.re3gistrycommon.model.Users;
import eu.europa.ec.re3gistrycommon.utils.PersistenceFactory;
import eu.europa.ec.re3gistrystaticizer.StaticizeFiles;
import eu.europa.ec.re3gistrystaticizer.util.Constants;
import java.io.IOException;
import java.util.Locale;
import java.util.ResourceBundle;
import javax.persistence.EntityManager;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.log4j.Logger;

public class StaticizerStart extends HttpServlet {

    private static final Logger log = Logger.getLogger(Constants.MODULE_NAME);
    private ResourceBundle localizationres;
    private EntityManager em;
    private DataprocedureMgr dataprocedureMgr;
    private Dataprocedure dataprocedure;
    private Users user;

    private void startProcedure(HttpServletRequest request, HttpServletResponse response) throws Exception {
        ReportMgr reportMgr = new ReportMgr();

        log.info("###");
        log.info("### Starting manual Staticization ###");

        /**
         * Localization settings
         */
        HttpSession s = request.getSession();
        String currentLanguage = (String) s.getAttribute(Constants.LANGUAGE_SESSION_KEY);
        if (currentLanguage == null) {
            currentLanguage = request.getLocale().toString();
        }

        /**
         * Initializing variables
         */
        this.localizationres = ResourceBundle.getBundle(Constants.LOCALIZATION_PREFIX + Constants.MODULE_NAME + Constants.LOCALIZATION_SUFFIX, new Locale(currentLanguage));
        this.em = PersistenceFactory.getEntityManagerFactory().createEntityManager();
        this.dataprocedureMgr = new DataprocedureMgr(em);
        this.user = (Users) request.getSession().getAttribute(eu.europa.ec.re3gistrycommon.constants.Constants.USER_KEY);

        String dataprocedureUUID = request.getParameter("id");

        this.dataprocedure = this.dataprocedureMgr.getDataprocedureByUuid(dataprocedureUUID);

        if (dataprocedureMgr.isProcedureInProgress(this.em)) {
            log.info("### The manual Staticization cannot be performed because another procedure is running. Try later.");
            s.setAttribute(Constants.KEY_SATICIZER_MANUAL_SESSION, localizationres.getString("error.staticizer.manual.run"));
        } else {
            StaticizeFiles staticizeFile = new StaticizeFiles(request, dataprocedure, reportMgr);
            staticizeFile.startToStaticizeFiles(this.dataprocedure, false);
        }
    }

    /**
     * Handles the HTTP <code>GET</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        try {
            startProcedure(request, response);
        } catch (Exception ex) {

        }
    }

    /**
     * Handles the HTTP <code>POST</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        try {
            startProcedure(request, response);
        } catch (Exception ex) {

        }
    }

    /**
     * Returns a short description of the servlet.
     *
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }
}
